@@ -1,9 +1,13 @@
-//  Config.cpp version 1.1
+//  Config.cpp version 1.5
 //  yudit package - Unicode Editor for the X Window System (and Linux) 
 //
 //  Author: gsinai@iname.com (Gaspar Sinai)
-//  GNU Copyright (C) 1997,1998  Gaspar Sinai
+//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
 // 
+//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
@@ -46,6 +50,7 @@ int		Config::xInputMapSize=0;
 Language**	Config::languageMap=0;
 int		Config::languageMapSize=0;
 char*		Config::typeIndex=0;
+
 static char*	trueTypeNames[UFreeType::WEIGHT_COUNT][UFreeType::SLANT_COUNT]=
 			{
 				{"Medium Roman", "Medium Italic"},
@@ -55,7 +60,8 @@ static char*	trueTypeNames[UFreeType::WEIGHT_COUNT][UFreeType::SLANT_COUNT]=
 char* 		Config::names[Config::CHECKLIST_MAX+1] = { 
 		"Encoding", "Input", "XInput", "Font", 
 		"Size", "Weight", "Slant",
-		"Spacing", "Tab Size", "Background", "Foreground", 
+		"Spacing", "Add.Style", "Avg.Width", 
+		"Tab Size", "Background", "Foreground", 
 		"Cursor", "Language", 0};
 
 static char*	xInputNames[XInputMap::MAX+1] = {
@@ -66,6 +72,14 @@ static char* 	languageMapNames[Language::MAX+1] = {
 	"Short Name", 
 	"Text Font", "Text Size", "Text Weight", "Text Slant", "Text Spacing",
 	"Button Font", "Button Size", "Button Weight", "Button Slant", "Button Spacing", 0};
+
+#define MAX_FALLBACK_ADD_STYLE  6
+static char* 	fallbackAddStyle[MAX_FALLBACK_ADD_STYLE] = { "Any", "sans", 
+			"serif", "fantizy", "jiantizi", "ja" };
+
+#define MAX_FALLBACK_AVG_WIDTH  9
+static char* 	fallbackAvgWidth[MAX_FALLBACK_AVG_WIDTH] = { "Any", "50", "60", 
+			"70", "80", "90", "100", "110", "120" };
 		
 
 enum LanguageIndex {SHORT_NAME, TEXT_FONT, TEXT_SIZE, 
@@ -118,6 +132,8 @@ Config::readConfig (const char* name, Mode mode)
 	AStringList	xArray;
 	int		xInputMissing;
 	int		xLanguageMissing;
+	int		addStyleMissing;
+	int		avgWidthMissing;
 	int		index;
 	AString		currentPrinter;
 
@@ -149,6 +165,8 @@ Config::readConfig (const char* name, Mode mode)
 	if (mode==REAL) path+=USE_PATH;
 	xInputMissing=0;
 	xLanguageMissing=0;
+	avgWidthMissing=0;
+	addStyleMissing=0;
 	for (i=0; (item=names[i])!=0; i++)
 	{
 		if (fileConfig->readEntry (item, (mode==TEST) ? testStr :  checked[i]) ==0)
@@ -165,6 +183,18 @@ Config::readConfig (const char* name, Mode mode)
 				cerr << "warn: [Default] Language is missing.\n";
 				continue;
 			}
+			if (strcmp (item, "Add.Style") == 0)
+			{
+				addStyleMissing=1;
+				cerr << "warn: [Default] Add.Style is missing.\n";
+				continue;
+			}
+			if (strcmp (item, "Avg.Width") == 0)
+			{
+				avgWidthMissing=1;
+				cerr << "warn: [Default] Avg.Width is missing.\n";
+				continue;
+			}
 			errors[mode] = "Error: config [Default] '";
 			errors[mode] += item;
 			errors[mode] += "' is missing. ";
@@ -249,6 +279,27 @@ Config::readConfig (const char* name, Mode mode)
 				cerr << "warn: [Menu Item] Language is missing.\n";
 				continue;
 			}
+			if (strcmp (item, "Add.Style") == 0)
+			{
+				cerr << "warn: [Menu Item] Add.Style is missing.\n";
+				if (mode != TEST)
+				{
+					checked[i] = (const char*) fallbackAddStyle[0];
+					checkList[i].copy ((const char**) fallbackAddStyle, 
+						MAX_FALLBACK_ADD_STYLE);
+				}
+				continue;
+			}
+			if (strcmp (item, "Avg.Width") == 0)
+			{
+				cerr << "warn: [Menu Item] Avg.Width is missing.\n";
+				if (mode != TEST)
+				{
+					checked[i] = (const char*) fallbackAvgWidth[0];
+					checkList[i].copy ((const char**) fallbackAvgWidth, MAX_FALLBACK_AVG_WIDTH);
+				}
+				continue;
+			}
 			
 			errors[mode] = "Error: config [Menu Item] '";
 			errors[mode] += item;
@@ -272,6 +323,22 @@ Config::readConfig (const char* name, Mode mode)
 			delete fileConfig;
 			return;
 		}
+		if (strcmp (item, "Add.Style") == 0 && addStyleMissing==1)
+		{
+			errors[mode] = "Error: config [Default] '";
+			errors[mode] += item;
+			errors[mode] += "' is missing. ";
+			delete fileConfig;
+			return;
+		}
+		if (strcmp (item, "Avg.Width") == 0 && avgWidthMissing==1)
+		{
+			errors[mode] = "Error: config [Default] '";
+			errors[mode] += item;
+			errors[mode] += "' is missing. ";
+			delete fileConfig;
+			return;
+		}
 	}
 	// go through fonts and get them into fontMaps
 	if (mode!=TEST)
@@ -465,6 +532,7 @@ Config::readConfig (const char* name, Mode mode)
 		if (mode!=TEST) fontMapSize++;
 		index++;
 	}
+
 	// go through XInputs and get them into xInputMap
 	if (mode!=TEST && checkList[XINPUT].getSize() > 0)
 	{
@@ -477,6 +545,7 @@ Config::readConfig (const char* name, Mode mode)
 		languageMap = new Language *[checkList[LANGUAGE].getSize()];
 		CHECKNULL (languageMap); 
 	}
+
 	for (i=0; i<((mode==TEST) ? 
 	   testArray[XINPUT].getSize() : checkList[XINPUT].getSize()); i++)
 	{
