@@ -1,9 +1,13 @@
-//  UFileDialog.cpp version 1.1
+//  UFileDialog.cpp version 1.5
 //  yudit package - Unicode Editor for the X Window System (and Linux) 
 //
 //  Author: gsinai@iname.com (Gaspar Sinai)
-//  GNU Copyright (C) 1997,1998  Gaspar Sinai
+//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
 // 
+//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
@@ -526,9 +530,12 @@ UFileDialog::eventUp (UEvent* event)
 	UTextMenu*	textMenu;
 	UBestSize	bs;
 	char*		str;
+	const char*		cstr;
 	int		i;
 	int		len;
 	const UCS2* const*	ucs2;
+	const int*	textSizes;
+	UCS2*		ustringVle;
 	UString		ustr;
 	ifstream*	ift;
 
@@ -551,22 +558,23 @@ UFileDialog::eventUp (UEvent* event)
 		{
 			selected = workingDir;
 			ucs2 = ((UTextFrame*)component[FILE_TEXT])->uGetTextBuffer();
-			if (ucs2==0 || ucs2[0]==0)
+			textSizes = ((UTextFrame*)component[FILE_TEXT])->uGetTextSizes();
+			if (ucs2==0 || textSizes[0] ==0) return;
+			if (UCS2NullLen(ucs2[0]) != textSizes[0])
 			{
+				cerr << "text contains null (^@) character.\n";
 				return;
 			}
-			if (ucs2!=0 && ucs2[0]!=0)
-			{
-				len = strlen ((const char*)selected);
-				// chop off '/'
-				if (len > 0) selected.truncate (len-1);
-				ustr.setConverter("UTF8");
-				ustr.putUString (ucs2[0]);
-				str = (char*) ustr.getString();
-				if (str[0]!='/') selected += "/";
-				selected += str;
-				delete str;
-			}
+			len = strlen ((const char*)selected);
+			// chop off '/'
+			if (len > 0) selected.truncate (len-1);
+			ustr.setConverter("UTF8");
+			ustr.putUString (ucs2[0], UCS2NullLen(ucs2[0]));
+			str = (char*) ustr.getString(&len);
+			if (str[0]!='/') selected += "/";
+			selected += str;
+			delete str;
+
 			// try open
 			ift = new ifstream ((const char*) selected);
 			if (type==LOAD)
@@ -598,10 +606,17 @@ UFileDialog::eventUp (UEvent* event)
 		if (event->client !=0 && event->client==component[CD])
 		{
 			ucs2 = ((UTextFrame*)component[FILE_TEXT])->uGetTextBuffer();
-			if (ucs2==0 || ucs2[0]==0) return;
+			textSizes = ((UTextFrame*)component[FILE_TEXT])->uGetTextSizes();
+			if (ucs2==0 || textSizes[0] ==0) return;
+			if (UCS2NullLen(ucs2[0]) != textSizes[0])
+			{
+				cerr << "text contains null (^@) character.\n";
+				return;
+			}
+			
 			ustr.setConverter("UTF8");
-			ustr.putUString (ucs2[0]);
-			str = (char*) ustr.getString();
+			ustr.putUString (ucs2[0], UCS2NullLen(ucs2[0]));
+			str = (char*) ustr.getString(&len);
 			if (cdTo ((const char*)str))
 			{
 				((UTextFrame*)component[FILE_TEXT])->uClear ();
@@ -629,10 +644,13 @@ UFileDialog::eventUp (UEvent* event)
 		if (event->client == component[FILE_LIST])
 		{
 			((UTextFrame*)component[FILE_TEXT])->uClear ();
-			UString str ((const unsigned char*) fileList.at (
-				event->value), "UTF8");
+			cstr = fileList.at (event->value);
+				
+			UString str ((const unsigned char*) cstr, "UTF8");
+			ustringVle = str.getUString (&len);
 			((UTextFrame*)component[FILE_TEXT])->uInsertUCS2(
-					(const UCS2*) str);
+					ustringVle, len);
+			delete ustringVle;
 			((UTextFrame*)component[FILE_TEXT])->uRefresh ();
 			((UTextFrame*)component[FILE_TEXT])->uClearModified();
 		}
