@@ -1,9 +1,13 @@
-//  MainWidget.cpp version 1.1
+//  MainWidget.cpp version 1.5
 //  yudit package - Unicode Editor for the X Window System (and Linux) 
 //
 //  Author: gsinai@iname.com (Gaspar Sinai)
-//  GNU Copyright (C) 1997,1998  Gaspar Sinai
+//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
 // 
+//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
@@ -81,6 +85,8 @@ MainWidget::MainWidget (const Config& configIn,
 	editFont->setWeight ((const char*)config.getChecked(Config::FONTWEIGHT));
 	editFont->setSlant ((const char*)config.getChecked(Config::FONTSLANT));
 	editFont->setSpacing ((const char*)config.getChecked(Config::FONTSPACING));
+	editFont->setAvgWidth ((const char*)config.getChecked(Config::FONT_AVG_WIDTH));
+	editFont->setAddStyle ((const char*)config.getChecked(Config::FONT_ADD_STYLE));
 
 	editArea->textFrame->uSetFont (editFont);
 	if (editArea->textFrame->uSetConverter(
@@ -284,6 +290,10 @@ MainWidget::buildMenu ()
 			UMenu::LEFT_BLANK, UMenu::RIGHT);
 	popUp[UFONTSPACING] = new UPopUp (popUp[UFONTNAME], "Spacing", Translate ("Spacing"), 
 			UMenu::LEFT_BLANK, UMenu::RIGHT);
+	popUp[UFONT_ADD_STYLE] = new UPopUp (popUp[UFONTNAME], "Add.Style", Translate ("Add.Style"), 
+			UMenu::LEFT_BLANK, UMenu::RIGHT);
+	popUp[UFONT_AVG_WIDTH] = new UPopUp (popUp[UFONTNAME], "Avg.Width", Translate ("Avg.Width"), 
+			UMenu::LEFT_BLANK, UMenu::RIGHT);
 	addButtons (UFONTSIZE, config.getCheckList (Config::FONTSIZE), UMenu::NO_ARROW);
 	checkItem (UFONTSIZE, config.getChecked (Config::FONTSIZE));
 
@@ -295,6 +305,10 @@ MainWidget::buildMenu ()
 
 	addButtons (UFONTSPACING, config.getCheckList (Config::FONTSPACING), UMenu::NO_ARROW);
 	checkItem (UFONTSPACING, config.getChecked (Config::FONTSPACING));
+	addButtons (UFONT_ADD_STYLE, config.getCheckList (Config::FONT_ADD_STYLE), UMenu::NO_ARROW);
+	checkItem (UFONT_ADD_STYLE, config.getChecked (Config::FONT_ADD_STYLE));
+	addButtons (UFONT_AVG_WIDTH, config.getCheckList (Config::FONT_AVG_WIDTH), UMenu::NO_ARROW);
+	checkItem (UFONT_AVG_WIDTH, config.getChecked (Config::FONT_AVG_WIDTH));
 
 
 	(void) new USeparator (popUp[UFONTNAME]);
@@ -413,13 +427,21 @@ MainWidget::translateMenu (MenuId id)
 	int		i;
 	UComponent*	comp;
 	UTextMenu*	tmenu;
+	UTextLabel*	tlabel;
 
 	for (i=0; i<popUp[id]->getSize(); i++)
 	{
 		comp = popUp[id]->itemAt (i);
-		if (!comp->isA (UComponent::TEXT_MENU)) continue;
-		tmenu = (UTextMenu*) comp;
-		tmenu->setText (Translate (tmenu->getName()));
+		if (comp->isA (UComponent::TEXT_MENU))
+		{
+			tmenu = (UTextMenu*) comp;
+			tmenu->setText (Translate (tmenu->getName()));
+		}
+		else if (comp->isA (UComponent::TEXT_LABEL))
+		{
+			tlabel = (UTextLabel*) comp;
+			tlabel->setText (Translate ("Not configured"));
+		}
 	}
 }
 
@@ -477,6 +499,8 @@ MainWidget::setXInput (const char* nameIn)
 
 // ---- these will appear in gettext ---
 // Translate ("Hungarian");
+// Translate ("German");
+// Translate ("Italian");
 // Translate ("English");
 // Translate ("Japanese");
 // Translate ("Chinese");
@@ -490,7 +514,7 @@ MainWidget::setXInput (const char* nameIn)
 // Translate ("Italic");
 // Translate ("Oblique");
 // Translate ("Monospace");
-// Translate ("Condensed");
+// Translate ("CharCell");
 // Translate ("Proportional");
 // Translate ("European");
 // Translate ("Kana");
