@@ -1,9 +1,13 @@
-//  MenuAction.cpp version 1.1
+//  MenuAction.cpp version 1.5
 //  yudit package - Unicode Editor for the X Window System (and Linux) 
 //
 //  Author: gsinai@iname.com (Gaspar Sinai)
-//  GNU Copyright (C) 1997,1998  Gaspar Sinai
+//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
 // 
+//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
@@ -254,6 +258,8 @@ MainWidget::menuAction (MenuId id, int menuItem)
 		editArea->textFrame->uSetFontSlant ((const char*)config.getChecked(Config::FONTSLANT));
 		editArea->textFrame->uSetFontWeight ((const char*)config.getChecked(Config::FONTWEIGHT));
 		editArea->textFrame->uSetFontSpacing ((const char*)config.getChecked(Config::FONTSPACING));
+		editArea->textFrame->uSetFontAvgWidth ((const char*)config.getChecked(Config::FONT_AVG_WIDTH));
+		editArea->textFrame->uSetFontAddStyle ((const char*)config.getChecked(Config::FONT_ADD_STYLE));
 		editArea->textFrame->uRefresh();
 		checkItem (UFONTNAME, str);
 		config.setChecked (Config::FONTNAME, str);
@@ -271,7 +277,27 @@ MainWidget::menuAction (MenuId id, int menuItem)
 		editFont->setWeight ((const char*)config.getChecked(Config::FONTWEIGHT));
 		editFont->setSlant ((const char*)config.getChecked(Config::FONTSLANT));
 		editFont->setSpacing ((const char*)config.getChecked(Config::FONTSPACING));
+		editFont->setAvgWidth ((const char*)config.getChecked(Config::FONT_AVG_WIDTH));
+		editFont->setAddStyle ((const char*)config.getChecked(Config::FONT_ADD_STYLE));
 
+		break;
+	case UFONT_ADD_STYLE:
+		if (findDialog) findDialog->hide ();
+		editArea->textFrame->uSetFontAddStyle (str);
+		if (editFont!=GetDefaultFont())
+			editFont->setAddStyle (str);
+		checkItem (UFONT_ADD_STYLE, str);
+		config.setChecked (Config::FONT_ADD_STYLE, str);
+		editArea->textFrame->uRefresh();
+		break;
+	case UFONT_AVG_WIDTH:
+		if (findDialog) findDialog->hide ();
+		editArea->textFrame->uSetFontAvgWidth (str);
+		if (editFont!=GetDefaultFont())
+			editFont->setAvgWidth (str);
+		checkItem (UFONT_AVG_WIDTH, str);
+		config.setChecked (Config::FONT_AVG_WIDTH, str);
+		editArea->textFrame->uRefresh();
 		break;
 	case UFONTSIZE:
 		if (findDialog) findDialog->hide ();
@@ -588,12 +614,15 @@ MainWidget::print ()
 	UString			titleString;
 	const UCS2* const*	text;
 	int			textSize;
-	const UCS2* 		header = 0;
+	const int*		textSizes;
+	UCS2* 			header = 0;
+	int 			headerLen = 0;
 	const char*		fontFile=0;
 	const char*		printerCommand=0;
 
 	text = editArea->textFrame->uGetTextBuffer();
 	textSize = editArea->textFrame->uGetTextSize ();
+	textSizes = editArea->textFrame->uGetTextSizes ();
 	if (text == 0 || textSize==0 || (textSize==1 && text[0][0]==0))
 	{
 		warn ("Nothing to print.\n");
@@ -747,12 +776,14 @@ MainWidget::print ()
 	if (!fileName.isNull())
 	{
 		titleString.putString ((const unsigned char*)((const char*) fileName));
-		header = (const UCS2*) titleString;
+		header = titleString.getUString(&headerLen);
 	}
 
 	// Here SGC
 	UPrint::UStatus status = uPrint->print (oStream, text,
-			textSize, header);
+			textSizes, textSize, header, headerLen);
+
+	if (header) delete header;
 
 	if (status!=UPrint::OK)
 	{
@@ -952,6 +983,7 @@ MainWidget::saveFile (const char* filenameIn)
 {
 	unsigned char*		text;
 	ofstream*		file;
+	int			textLen;
 	int			error;
 
 	if (strlen(filenameIn) > strlen ("file:") 
@@ -972,10 +1004,10 @@ MainWidget::saveFile (const char* filenameIn)
 		return -1;
 	}
 	error=0;
-	text = editArea->textFrame->uGetBuffer();
+	text = editArea->textFrame->uGetBuffer(&textLen);
 	if (text !=0)
 	{
-		if (UWriteBuffer(file, text) ==-1) error=1;
+		if (UWriteBuffer(file, text, textLen) ==-1) error=1;
 		delete text;
 	}
 	file->close();
@@ -1032,6 +1064,7 @@ MainWidget::appendFileBase (const char* filenameIn)
 {
 	ifstream*	file;
 	unsigned char* 	buffer;
+	int		bufferLen;
 
 	if (strlen(filenameIn) > strlen ("file:") 
 		&& strncmp (filenameIn, "file:", strlen ("file:")) == 0)
@@ -1053,8 +1086,8 @@ MainWidget::appendFileBase (const char* filenameIn)
 		delete file;
 		return -1;
 	}
-	buffer = UReadBuffer (file);
-	editArea->textFrame->uAppendText (buffer);
+	buffer = UReadBuffer (file, &bufferLen);
+	editArea->textFrame->uAppendText (buffer, bufferLen, 0, 0);
 	delete buffer;
 	file->close ();
 	delete file;
