@@ -1,9 +1,13 @@
-//  UCS2Conv.cpp version 1.1
+//  UCS2Conv.cpp version 1.5
 //  yudit package - Unicode Editor for the X Window System (and Linux) 
 //
 //  Author: gsinai@iname.com (Gaspar Sinai)
-//  GNU Copyright (C) 1997,1998  Gaspar Sinai
+//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
 // 
+//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
@@ -93,10 +97,17 @@ UCS2Converter::~UCS2Converter ()
 }
 
 void
-UCS2Converter::init (const UCS2* input)
+UCS2Converter::init (const UCS2* input, int _size)
 {
-	inputSize = 0;
-	while (input[inputSize]!=0) inputSize++;
+	if (_size < 0)
+	{
+		inputSize = 0;
+		while (input[inputSize]!=0) inputSize++;
+	}
+	else
+	{
+		inputSize = _size;
+	}
 	bufferSize = inputSize +2;
 	size = 0;
 	uchar = new unsigned char [bufferSize];
@@ -105,10 +116,17 @@ UCS2Converter::init (const UCS2* input)
 }
 
 void
-UCS2Converter::init (const unsigned char* input)
+UCS2Converter::init (const unsigned char* input, int _size)
 {
-	inputSize = 0;
-	while (input[inputSize]!=0) inputSize++;
+	if (_size > 0)
+	{
+		inputSize = _size;
+	}
+	else
+	{
+		inputSize = 0;
+		while (input[inputSize]!=0) inputSize++;
+	}
 	bufferSize = inputSize +2;
 	size = 0;
 	ucs2 = new UCS2 [bufferSize];
@@ -135,6 +153,13 @@ UCS2Converter::append (const UCS2 in)
 {
 	UCS2*		newBuffer;
 
+/*
+	if (in ==0)
+	{
+		appendQuoted ((UCS2) in);
+		return;
+	}
+*/
 	if (size>=bufferSize)
 	{
 		bufferSize += bufferSize/2;
@@ -146,6 +171,27 @@ UCS2Converter::append (const UCS2 in)
 	ucs2[size++] = (UCS2) in;
 }
 
+void
+UCS2Converter::terminateStr ()
+{
+	append ((unsigned char) 0);
+}
+
+void
+UCS2Converter::terminateUCS2 ()
+{
+	UCS2*		newBuffer;
+	if (size>=bufferSize)
+	{
+		bufferSize += bufferSize/2;
+		newBuffer = new UCS2 [bufferSize];
+		memcpy (newBuffer, ucs2, size * sizeof (UCS2));
+		delete ucs2;
+		ucs2 = newBuffer;
+	}
+	ucs2[size++] = (UCS2) 0;
+}
+
 //
 // append into UCS2
 //
@@ -167,6 +213,23 @@ UCS2Converter::append32 (const QUOTE32 ucs4)
 	append ((UCS2) hexmap[ucs4&0xf]);
 }
 
+//
+// append into UCS2
+//
+void
+UCS2Converter::appendQuoted (const UCS2 u2)
+{
+	char 		hexmap[] = {'0', '1', '2', '3', 
+			'4', '5', '6', '7', '8', '9', 
+			'a', 'b', 'c', 'd', 'e', 'f'};
+	append ((UCS2) '\\');
+	append ((UCS2) 'u');
+	append ((UCS2) hexmap[(u2>>12)&0xf]);
+	append ((UCS2) hexmap[(u2>>8)&0xf]);
+	append ((UCS2) hexmap[(u2>>4)&0xf]);
+	append ((UCS2) hexmap[u2&0xf]);
+}
+
 //
 // append into char
 //
@@ -204,39 +267,60 @@ UCS2Converter::append8 (const QUOTE8 quote)
 // ucs4 characters in th range of 0x00010000-0xffffffff
 // are translated into ucs2 by \Uxxxxxxxx
 UCS2*
-UCS2Converter::decode (const unsigned char *in)
+UCS2Converter::decode (const unsigned char *in, int _size, int *retSize)
 {
 	int		i;
 	UCS2		decoded;
 	QUOTE32		quoted;
 
-	init (in);
+	init (in, _size);
 
 	for (i=0; i<inputSize; i++)
 	{
-		if ((in[i] & 0xe0) ==0xc0 && inputSize-i > 1 && (in[i+1] & 0x40)==0 )
+		// Unexpected continuation bytes
+		if (in[i] <= 0xbf && in[i] >= 0x80)
+		{
+			append8 (in[i]);
+			continue;
+		}
+
+		if ((in[i] & 0xe0) ==0xc0 && inputSize-i > 1 && (in[i+1] & 0xc0)==0x80 )
 		{
 			// check - the second 
 			decoded = (((unsigned short)(in[i] & 0x1f)) << 6)
 					| ((unsigned short) (in[i+1] & 0x3f));
-			append (decoded);
+			if (decoded < 0x80)
+			{
+				appendQuoted ((UCS2) decoded);
+			}
+			else
+			{
+				append (decoded);
+			}
 			i++;
 			continue;
 		}
 		if ((in[i] & 0xf0)==0xe0 && inputSize-i > 2
-			&& (in[i+1] & 0x40)==0 && (in[i+2] & 0x40)==0)
+			&& (in[i+1] & 0xc0)==0x80 && (in[i+2] & 0xc0)==0x80)
 		{
 			decoded = (((unsigned short) (in[i] & 0x0f)) << 12)
 					| (((unsigned short) (in[i+1] & 0x3f))<<6)
 					| ((unsigned short) (in[i+2] & 0x3f));
-			append (decoded);
+			if (decoded < 0x800)
+			{
+				appendQuoted ((UCS2) decoded);
+			}
+			else
+			{
+				append (decoded);
+			}
 			i++;
 			i++;
 			continue;
 		}
 		if ((in[i] & 0xf8)==0xf0 && inputSize-i > 3
-			&& (in[i+1] & 0x40)==0 && (in[i+2] & 0x40)==0 
-			&& (in[i+3] & 0x40)==0)
+			&& (in[i+1] & 0xc0)==0x80 && (in[i+2] & 0xc0)==0x80 
+			&& (in[i+3] & 0xc0)==0x80)
 		{
 			quoted = (((unsigned int) (in[i] & 0x07)) << 18)
 				| (((unsigned int) (in[i+1] & 0x3f))<<12)
@@ -249,11 +333,11 @@ UCS2Converter::decode (const unsigned char *in)
 			continue;
 		}
 		if ((in[i] & 0xfc)==0xf8 && inputSize-i > 4
-			&& (in[i+1] & 0x40)==0 && (in[i+2] & 0x40)==0 
-			&& (in[i+3] & 0x40)==0 && (in[i+4] & 0x40)==0)
+			&& (in[i+1] & 0xc0)==0x80 && (in[i+2] & 0xc0)==0x80 
+			&& (in[i+3] & 0xc0)==0x80 && (in[i+4] & 0xc0)==0x80)
 		{
 			quoted = (((unsigned int) (in[i] & 0x03)) << 24)
-				| (((unsigned int) (in[i+1] & 0x0f)) << 18)
+				| (((unsigned int) (in[i+1] & 0x3f)) << 18)
 				| (((unsigned int) (in[i+2] & 0x3f))<<12)
 				| (((unsigned short) (in[i+3] & 0x3f))<<6)
 				| ((unsigned short) (in[i+4] & 0x3f));
@@ -265,9 +349,9 @@ UCS2Converter::decode (const unsigned char *in)
 			continue;
 		}
 		if ((in[i] & 0xfe)==0xfc && inputSize-i > 5
-			&& (in[i+1] & 0x40)==0 && (in[i+2] & 0x40)==0 
-			&& (in[i+3] & 0x40)==0 && (in[i+4] & 0x40)==0
-			&& (in[i+5] & 0x40)==0)
+			&& (in[i+1] & 0xc0)==0x80 && (in[i+2] & 0xc0)==0x80 
+			&& (in[i+3] & 0xc0)==0x80 && (in[i+4] & 0xc0)==0x80
+			&& (in[i+5] & 0xc0)==0x80)
 		{
 			quoted =  (((unsigned int) (in[i] & 0x01)) << 30)
 				| (((unsigned int) (in[i+1] & 0x3f)) << 24)
@@ -284,23 +368,31 @@ UCS2Converter::decode (const unsigned char *in)
 			continue;
 		}
 
+		if (in[i] >= 0x80)
+		{
+			append8 (in[i]);
+			continue;
+		}
+
 		// we translate broken utf8 into ucs2 also...
 		append ((UCS2) in[i]);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 //
 //
 unsigned char*
-UCS2Converter::encode (const UCS2 *in)
+UCS2Converter::encode (const UCS2 *in, int _size, int* retSize)
 {
 	int	i;
 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
+
 		// The order is important.
 		if (in[i] >= 0x0800)
 		{
@@ -317,10 +409,8 @@ UCS2Converter::encode (const UCS2 *in)
 		}
 		append ((unsigned char) in[i]);
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -358,12 +448,12 @@ UUTF7Conv::~UUTF7Conv ()
 }
 
 UCS2*
-UUTF7Conv::decode (const unsigned char* in)
+UUTF7Conv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	int		i;
 	int		start;
 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (inputSize-i > 1 && in[i] == '+' && in[i+1] == '-')
@@ -388,17 +478,18 @@ UUTF7Conv::decode (const unsigned char* in)
 		if (in[i] != '-') i--;
 		mdecode (start);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-UUTF7Conv::encode (const UCS2 *in)
+UUTF7Conv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	int		i;
 	int		start;
 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80 && allowedChars[in[i]]==1)
@@ -440,10 +531,8 @@ UUTF7Conv::encode (const UCS2 *in)
 		if (in[i] == 0) break;
 		i--;
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -556,12 +645,12 @@ UJavaConv::~UJavaConv ()
 }
 
 unsigned char*
-UJavaConv::encode (const UCS2 *in)
+UJavaConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i]<0x80)
@@ -571,22 +660,20 @@ UJavaConv::encode (const UCS2 *in)
 		}
 		append16 ((const QUOTE16) in[i]);
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
 UCS2*
-UJavaConv::decode (const unsigned char *in)
+UJavaConv::decode (const unsigned char *in, int _size, int* retSize)
 {
 	int		i;
 	UCS2		decoded;
 	QUOTE32		quoted;
 	unsigned char*	toOut;
 
-	init (in);
+	init (in, _size);
 
 	for (i=0; i<inputSize; i++)
 	{
@@ -603,30 +690,53 @@ UJavaConv::decode (const unsigned char *in)
 			}
 			// life goes on..
 		}
+		//
 		// It tries as UTF8
-		if ((in[i] & 0xe0) ==0xc0 && inputSize-i > 1 && (in[i+1] & 0x40)==0 )
+		//
+		// Unexpected continuation bytes
+		if (in[i] <= 0xbf && in[i] >= 0x80)
+		{
+			append8 (in[i]);
+			continue;
+		}
+
+		if ((in[i] & 0xe0) ==0xc0 && inputSize-i > 1 && (in[i+1] & 0xc0)==0x80 )
 		{
 			// check - the second 
 			decoded = (((unsigned short)(in[i] & 0x1f)) << 6)
 					| ((unsigned short) (in[i+1] & 0x3f));
-			append (decoded);
+			if (decoded < 0x80)
+			{
+				appendQuoted ((UCS2) decoded);
+			}
+			else
+			{
+				append (decoded);
+			}
 			i++;
 			continue;
 		}
 		if ((in[i] & 0xf0)==0xe0 && inputSize-i > 2
-			&& (in[i+1] & 0x40)==0 && (in[i+2] & 0x40)==0)
+			&& (in[i+1] & 0xc0)==0x80 && (in[i+2] & 0xc0)==0x80)
 		{
 			decoded = (((unsigned short) (in[i] & 0x0f)) << 12)
 					| (((unsigned short) (in[i+1] & 0x3f))<<6)
 					| ((unsigned short) (in[i+2] & 0x3f));
-			append (decoded);
+			if (decoded < 0x800)
+			{
+				appendQuoted ((UCS2) decoded);
+			}
+			else
+			{
+				append (decoded);
+			}
 			i++;
 			i++;
 			continue;
 		}
 		if ((in[i] & 0xf8)==0xf0 && inputSize-i > 3
-			&& (in[i+1] & 0x40)==0 && (in[i+2] & 0x40)==0 
-			&& (in[i+3] & 0x40)==0)
+			&& (in[i+1] & 0xc0)==0x80 && (in[i+2] & 0xc0)==0x80 
+			&& (in[i+3] & 0xc0)==0x80)
 		{
 			quoted = (((unsigned int) (in[i] & 0x07)) << 18)
 				| (((unsigned int) (in[i+1] & 0x3f))<<12)
@@ -639,8 +749,8 @@ UJavaConv::decode (const unsigned char *in)
 			continue;
 		}
 		if ((in[i] & 0xfc)==0xf8 && inputSize-i > 4
-			&& (in[i+1] & 0x40)==0 && (in[i+2] & 0x40)==0 
-			&& (in[i+3] & 0x40)==0 && (in[i+4] & 0x40)==0)
+			&& (in[i+1] & 0xc0)==0x80 && (in[i+2] & 0xc0)==0x80 
+			&& (in[i+3] & 0xc0)==0x80 && (in[i+4] & 0xc0)==0x80)
 		{
 			quoted = (((unsigned int) (in[i] & 0x03)) << 24)
 				| (((unsigned int) (in[i+1] & 0x0f)) << 18)
@@ -655,9 +765,9 @@ UJavaConv::decode (const unsigned char *in)
 			continue;
 		}
 		if ((in[i] & 0xfe)==0xfc && inputSize-i > 5
-			&& (in[i+1] & 0x40)==0 && (in[i+2] & 0x40)==0 
-			&& (in[i+3] & 0x40)==0 && (in[i+4] & 0x40)==0
-			&& (in[i+5] & 0x40)==0)
+			&& (in[i+1] & 0xc0)==0x80 && (in[i+2] & 0xc0)==0x80 
+			&& (in[i+3] & 0xc0)==0x80 && (in[i+4] & 0xc0)==0x80
+			&& (in[i+5] & 0xc0)==0x80)
 		{
 			quoted =  (((unsigned int) (in[i] & 0x01)) << 30)
 				| (((unsigned int) (in[i+1] & 0x3f)) << 24)
@@ -674,10 +784,16 @@ UJavaConv::decode (const unsigned char *in)
 			continue;
 		}
 
+		if (in[i] >= 0x80)
+		{
+			append8 (in[i]);
+			continue;
+		}
 		// we translate broken utf8 into ucs2 also...
 		append ((UCS2) in[i]);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
@@ -741,7 +857,7 @@ UJISConv::initialize ()
 }
 
 UCS2*
-UJISConv::decode (const unsigned char* in)
+UJISConv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UMap		*current=0;
 	UCS2		got;
@@ -749,7 +865,7 @@ UJISConv::decode (const unsigned char* in)
 	int		katakana=0;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	current = 0;
 	for (i=0; i<inputSize; i++)
 	{
@@ -852,7 +968,8 @@ UJISConv::decode (const unsigned char* in)
 		append ((UCS2) in[i]);
 		continue;
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
@@ -860,7 +977,7 @@ UJISConv::decode (const unsigned char* in)
 // Hmm.... ESC [$] ) .. dollar is missing here...
 //
 unsigned char*
-UJISConv::encode (const UCS2 *in)
+UJISConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	UMap		*current=0;
@@ -868,7 +985,7 @@ UJISConv::encode (const UCS2 *in)
 	int		katakana=0;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -964,10 +1081,8 @@ UJISConv::encode (const UCS2 *in)
 		append ((unsigned char) '(');
 		append ((unsigned char) 'B');
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -1013,7 +1128,7 @@ USJISConv::initialize ()
 }
 
 UCS2*
-USJISConv::decode (const unsigned char* in)
+USJISConv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UCS2		got;
 	UCS2		code;
@@ -1023,7 +1138,7 @@ USJISConv::decode (const unsigned char* in)
 	unsigned short	cellOffset;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		// Half-width katakana
@@ -1072,19 +1187,60 @@ USJISConv::decode (const unsigned char* in)
 			continue;
 		} 
 
-		if (in[i] > 0x7e)
+		// User defined area
+		if (inputSize -i > 1 && 
+			((in[i] >= 0xf0 && in[i] <= 0xfc) 
+			  || (in[i] >= 0x40 && in[i] <= 0x7e))
+		 && 
+			((in[i] >= 0xf0 && in[i] <= 0xfc) 
+			  || (in[i+1] >= 0x80 && in[i+1] <= 0xfc)) )
+			
+		{
+			append8 ((QUOTE8) in[i]);
+			append8 ((QUOTE8) in[i+1]);
+			i++;
+			continue;
+		}
+
+		// MAC - backslash
+		if (in[i] == 0x80)
+		{
+			append ((UCS2) '\\');
+			continue;
+		}
+		// MAC - copyright
+		if (in[i] == 0xfd)
+		{
+			append ((UCS2) 0xa9);
+			continue;
+		}
+		// MAC - tm
+		if (in[i] == 0xfe)
+		{
+			append ((UCS2) 0x2122);
+			continue;
+		}
+		// MAC - ... horizontal ellipsis
+		if (in[i] == 0xff)
+		{
+			append ((UCS2) 0x2026);
+			continue;
+		}
+
+		if (in[i] > 0x80)
 		{
 			append8 ((QUOTE8) in[i]);
 			continue;
 		}
 		append ((UCS2) in[i]);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-USJISConv::encode (const UCS2 *in)
+USJISConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	unsigned short	rowOffset;
@@ -1094,7 +1250,7 @@ USJISConv::encode (const UCS2 *in)
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -1135,10 +1291,8 @@ USJISConv::encode (const UCS2 *in)
 		}
 		append16 ((const QUOTE16) in[i]);
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -1153,13 +1307,13 @@ UEUC_JPConv::~UEUC_JPConv ()
 }
 
 UCS2*
-UEUC_JPConv::decode (const unsigned char* in)
+UEUC_JPConv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		got=0;
@@ -1231,18 +1385,19 @@ UEUC_JPConv::decode (const unsigned char* in)
 		} 
 		append ((UCS2) in[i]);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-UEUC_JPConv::encode (const UCS2 *in)
+UEUC_JPConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -1284,10 +1439,8 @@ UEUC_JPConv::encode (const UCS2 *in)
 		}
 		append16 ((const QUOTE16) in[i]);
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -1302,7 +1455,7 @@ CTEXT_JAConv::~CTEXT_JAConv ()
 }
 
 UCS2*
-CTEXT_JAConv::decode (const unsigned char* in)
+CTEXT_JAConv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UMap		*current=0;
 	UCS2		got;
@@ -1310,7 +1463,7 @@ CTEXT_JAConv::decode (const unsigned char* in)
 	int		right;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	current = 0;
 	right = 0;
 	for (i=0; i<inputSize; i++)
@@ -1347,6 +1500,17 @@ CTEXT_JAConv::decode (const unsigned char* in)
 			i++; i++; i++; 
 			continue;
 		} 
+		//
+		// Kterm has the habit of setting GR instead of GL
+		//
+		if (inputSize -i > 3 && in[i] == ESC && in[i+1] == '$' 
+			&& in[i+2] == ')' && in[i+3] == 'D')
+		{
+			current=mJIS0212; // JIS X JIS0212-1990
+			right = 0;
+			i++; i++; i++; 
+			continue;
+		} 
 		if (inputSize -i > 2 && in[i] == ESC  && in[i+1] == '(' 
 			&& in[i+2] == 'B')
 		{
@@ -1468,12 +1632,13 @@ CTEXT_JAConv::decode (const unsigned char* in)
 		append ((UCS2) in[i]);
 		continue;
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-CTEXT_JAConv::encode (const UCS2 *in)
+CTEXT_JAConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	UMap		*current=0;
@@ -1481,7 +1646,7 @@ CTEXT_JAConv::encode (const UCS2 *in)
 	int		katakana=0;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -1581,10 +1746,8 @@ CTEXT_JAConv::encode (const UCS2 *in)
 		append ((unsigned char) '(');
 		append ((unsigned char) 'B');
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -1629,7 +1792,7 @@ UGB2312_7Conv::initialize ()
 // 7 Bit GB2312_7
 //
 UCS2*
-UGB2312_7Conv::decode (const unsigned char* in)
+UGB2312_7Conv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UMap		*current=0;
 	UCS2		got;
@@ -1637,7 +1800,7 @@ UGB2312_7Conv::decode (const unsigned char* in)
 	int		gbRoman=0;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	current = 0;
 	for (i=0; i<inputSize; i++)
 	{
@@ -1690,19 +1853,20 @@ UGB2312_7Conv::decode (const unsigned char* in)
 		append ((UCS2) in[i]);
 		continue;
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-UGB2312_7Conv::encode (const UCS2 *in)
+UGB2312_7Conv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	UMap		*current=0;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -1745,10 +1909,8 @@ UGB2312_7Conv::encode (const UCS2 *in)
 		append ((unsigned char) '(');
 		append ((unsigned char) 'B');
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -1764,13 +1926,13 @@ UGB2312_8Conv::~UGB2312_8Conv ()
 }
 
 UCS2*
-UGB2312_8Conv::decode (const unsigned char* in)
+UGB2312_8Conv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		got=0;
@@ -1798,18 +1960,19 @@ UGB2312_8Conv::decode (const unsigned char* in)
 		} 
 		append ((UCS2) in[i]);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-UGB2312_8Conv::encode (const UCS2 *in)
+UGB2312_8Conv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -1830,10 +1993,8 @@ UGB2312_8Conv::encode (const UCS2 *in)
 		}
 		append16 ((const QUOTE16) in[i]);
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -1850,14 +2011,14 @@ UHZConv::~UHZConv ()
 }
 
 UCS2*
-UHZConv::decode (const unsigned char* in)
+UHZConv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UCS2		got;
 	UMap		*current=0;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		got=0;
@@ -1900,19 +2061,20 @@ UHZConv::decode (const unsigned char* in)
 		}
 		append ((UCS2) in[i]);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-UHZConv::encode (const UCS2 *in)
+UHZConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	UMap		*current=0;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -1953,10 +2115,8 @@ UHZConv::encode (const UCS2 *in)
 		append ((unsigned char) '}');
 		current=0;
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -2001,14 +2161,14 @@ UKSC5601Conv::initialize ()
 // 7 Bit KSC5601
 //
 UCS2*
-UKSC5601Conv::decode (const unsigned char* in)
+UKSC5601Conv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UMap		*current=0;
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	current = 0;
 	for (i=0; i<inputSize; i++)
 	{
@@ -2067,19 +2227,20 @@ UKSC5601Conv::decode (const unsigned char* in)
 		append ((UCS2) in[i]);
 		continue;
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-UKSC5601Conv::encode (const UCS2 *in)
+UKSC5601Conv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	UMap		*current=0;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -2121,10 +2282,8 @@ UKSC5601Conv::encode (const UCS2 *in)
 		append ((unsigned char) '(');
 		append ((unsigned char) 'B');
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -2140,13 +2299,13 @@ UEUC_KRConv::~UEUC_KRConv ()
 }
 
 UCS2*
-UEUC_KRConv::decode (const unsigned char* in)
+UEUC_KRConv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		got=0;
@@ -2174,18 +2333,19 @@ UEUC_KRConv::decode (const unsigned char* in)
 		} 
 		append ((UCS2) in[i]);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-UEUC_KRConv::encode (const UCS2 *in)
+UEUC_KRConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -2206,10 +2366,8 @@ UEUC_KRConv::encode (const UCS2 *in)
 		}
 		append16 ((const QUOTE16) in[i]);
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -2225,24 +2383,28 @@ UUHCConv::~UUHCConv ()
 }
 
 UCS2*
-UUHCConv::decode (const unsigned char* in)
+UUHCConv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		got=0;
-		if (inputSize -i > 1 && ( in[i] > 0xa0 && in[i] < 0xff
-			&& in[i+1] > 0xa0 && in[i+1] < 0xff)   ||
-			( in[i] > 0x80 && in[i] < 0xc6  &&   
-			  (in[i+1] > 0x40 && in[i+1] < 0x5b ||
-			     in[i+1] > 0x60 && in[i+1] < 0x7b )  ) ||
-			( in[i] == 0xc6 && in[i+1] > 0x40 && in[i+1] < 0x53) )
+		//
+		// patch from Jungshik Shin 9 Nov 1998
+		//
+                if (inputSize -i > 1 && ( in[i] > 0xa0 && in[i] < 0xff
+                        && in[i+1] > 0xa0 && in[i+1] < 0xff)     ||
+                        ( in[i] > 0x80 && in[i] < 0xc6  &&   
+                          (in[i+1] > 0x40 && in[i+1] < 0x5b ||
+                           in[i+1] > 0x60 && in[i+1] < 0x7b ||
+                           in[i+1] > 0x80 && in[i+1] < 0xff  ) ) ||
+                        ( in[i] == 0xc6 && in[i+1] > 0x40 && in[i+1] < 0x53) )
 
-		{
+                {
 			// KSC5601
 			if (mKSC5601!=0)
 			{
@@ -2264,18 +2426,19 @@ UUHCConv::decode (const unsigned char* in)
 		} 
 		append ((UCS2) in[i]);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-UUHCConv::encode (const UCS2 *in)
+UUHCConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -2291,10 +2454,8 @@ UUHCConv::encode (const UCS2 *in)
 		}
 		append16 ((const QUOTE16) in[i]);
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -2312,7 +2473,7 @@ UJOHABConv::~UJOHABConv ()
 
 
 UCS2*
-UJOHABConv::decode (const unsigned char* in)
+UJOHABConv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UCS2		got;
 	UCS2		code;
@@ -2357,7 +2518,7 @@ UJOHABConv::decode (const unsigned char* in)
 
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 
 	for (i=0; i<inputSize; i++)
 	{
@@ -2461,12 +2622,13 @@ UJOHABConv::decode (const unsigned char* in)
 		}
 		append ((UCS2) in[i]);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-UJOHABConv::encode (const UCS2 *in)
+UJOHABConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	unsigned char	c1;
@@ -2491,7 +2653,7 @@ UJOHABConv::encode (const UCS2 *in)
 
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -2547,10 +2709,8 @@ UJOHABConv::encode (const UCS2 *in)
 		}
 		append16 ((const QUOTE16) in[i]);
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -2583,13 +2743,13 @@ UBIG5Conv::initialize ()
 }
 
 UCS2*
-UBIG5Conv::decode (const unsigned char* in)
+UBIG5Conv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		got=0;
@@ -2617,18 +2777,19 @@ UBIG5Conv::decode (const unsigned char* in)
 		} 
 		append ((UCS2) in[i]);
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
 unsigned char*
-UBIG5Conv::encode (const UCS2 *in)
+UBIG5Conv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	for (i=0; i<inputSize; i++)
 	{
 		if (in[i] < 0x80)
@@ -2649,10 +2810,8 @@ UBIG5Conv::encode (const UCS2 *in)
 		}
 		append16 ((const QUOTE16) in[i]);
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -2692,7 +2851,7 @@ U8BitConv::initialize ()
 // In case it is 16bit mapped we use Big endian.
 //
 UCS2*
-U8BitConv::decode (const unsigned char* in)
+U8BitConv::decode (const unsigned char* in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
@@ -2700,21 +2859,21 @@ U8BitConv::decode (const unsigned char* in)
 	UCS2		value;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 	step = (map !=0 && map->encodeResultMax() > 0xff) ? 2 : 1;
-	for (i=0; !(in[i]==0 && in[i+1]==0); i=i+step)
+	for (i=0; i<inputSize-step+1; i=i+step)
 	{
 		value = (step==1) ? in[i] : ((in[i]<<8) | in[i+1]); 
-		if (value<0x80)
-		{
-			append (value);
-			continue;
-		}
 		if (map != 0 && (got=map->decode (value)) != 0)
 		{
 			append ((UCS2) got);
 			continue;
 		}
+		if (value<0x80)
+		{
+			append (value);
+			continue;
+		}
 		if (step==1)
 		{
 			append8 ((QUOTE8) value);
@@ -2724,7 +2883,8 @@ U8BitConv::decode (const unsigned char* in)
 			append ((UCS2) '?');
 		}
 	}
-	append ((UCS2) 0);
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
@@ -2734,30 +2894,30 @@ U8BitConv::decode (const unsigned char* in)
 // In case it is 16bit mapped we use Big endian.
 //
 unsigned char*
-U8BitConv::encode (const UCS2 *in)
+U8BitConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	UCS2		got;
 	int		i;
 	int		step;
 
 	initialize (); 
-	init (in);
+	init (in, _size);
 
 	step = (map !=0 && map->encodeResultMax() > 0xff) ? 2 : 1;
 	for (i=0; i<inputSize; i++)
 	{
-		if (in[i]<0x80)
-		{
-			if (step == 2) append ((unsigned char) 0);
-			append ((unsigned char)in[i]);
-			continue;
-		}
 		if (map != 0 && (got=map->encode (in[i])) != 0)
 		{
 			if (step == 2) append ((unsigned char) (got>>8));
 			append ((unsigned char) (got&0xff));
 			continue;
 		}
+		if (in[i]<0x80)
+		{
+			if (step == 2) append ((unsigned char) 0);
+			append ((unsigned char)in[i]);
+			continue;
+		}
 		if (step == 2)
 		{
 			append ((unsigned char) 0);
@@ -2768,10 +2928,8 @@ U8BitConv::encode (const UCS2 *in)
 			append16 ((const QUOTE16) in[i]);
 		}
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -2786,13 +2944,13 @@ U16BitConv::~U16BitConv()
 }
 
 unsigned char*
-U16BitConv::encode (const UCS2 *in)
+U16BitConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	int		i;
 	unsigned char	ucs[2];
-	init 		(in);
+	init 		(in, _size);
 
-	for (i=0; in[i]!=0; i++)
+	for (i=0; i<inputSize; i++)
 	{
 		// 12 34 -> 12 34
 		if (endian==UCS2BE)
@@ -2809,21 +2967,19 @@ U16BitConv::encode (const UCS2 *in)
 		append (ucs[0]);
 		append (ucs[1]);
 	}
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
 UCS2*
-U16BitConv::decode (const unsigned char *in)
+U16BitConv::decode (const unsigned char *in, int _size, int* retSize)
 {
 	int	i;
 	UCS2	ucs;
 
-	init (in);
-	for (i=0; !(in[i]==0 && in[i+1]==0); i=i+2)
+	init (in, _size);
+	for (i=0; i<inputSize-1; i=i+2)
 	{
 		if (endian==UCS2BE)
 		{
@@ -2835,7 +2991,9 @@ U16BitConv::decode (const unsigned char *in)
 		}
 		append (ucs);
 	}
-	append ((UCS2) 0);
+	if (inputSize % 2 != 0) append ((UCS2) '?');
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
@@ -2871,11 +3029,11 @@ UUniConv::~UUniConv()
 // It puts \r at the end of line.
 //
 unsigned char*
-UUniConv::encode (const UCS2 *in)
+UUniConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	int		i;
 	unsigned char	ucs[2];
-	init 		(in);
+	init 		(in, _size);
 
 	endian = UCS2BE;
 	if (endian==UCS2BE)
@@ -2891,7 +3049,7 @@ UUniConv::encode (const UCS2 *in)
 
 	append (ucs[0]);
 	append (ucs[1]);
-	for (i=0; in[i]!=0; i++)
+	for (i=0; i<inputSize; i++)
 	{
 		// 12 34 -> 12 34
 		if (endian==UCS2BE)
@@ -2907,11 +3065,8 @@ UUniConv::encode (const UCS2 *in)
 		append (ucs[0]);
 		append (ucs[1]);
 	}
-
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -2920,17 +3075,16 @@ UUniConv::encode (const UCS2 *in)
 // line  separator paragraph seperator and \r is 
 // translated.
 UCS2*
-UUniConv::decode (const unsigned char *in)
+UUniConv::decode (const unsigned char *in, int _size, int* retSize)
 {
 	int	i;
 	UCS2	ucs;
 
-	init (in);
+	init (in, _size);
 
 	// set some defualt state.
 	endian = UCS2BE;
-	for (i=0; !(in[i]==0 && in[i+1]==0);
-		i=i+2)
+	for (i=0; i<inputSize-1; i=i+2)
 	{
 		if (in[i] == (UNI_MAGIC>>8) 
 			&& in[i+1] == (UNI_MAGIC&0xff))
@@ -2955,7 +3109,9 @@ UUniConv::decode (const unsigned char *in)
 		append (ucs);
 		
 	}
-	append ((UCS2) 0);
+	if (inputSize % 2 != 0) append ((UCS2) '?');
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
 
@@ -2972,12 +3128,12 @@ UMSTXTConv::~UMSTXTConv()
 // It puts \r at the end of line.
 //
 unsigned char*
-UMSTXTConv::encode (const UCS2 *in)
+UMSTXTConv::encode (const UCS2 *in, int _size, int* retSize)
 {
 	int		i;
 	unsigned char	ucs[2];
 	unsigned char	ucsr[2];
-	init 		(in);
+	init 		(in, _size);
 
 	endian = UCS2LE;
 	if (endian==UCS2BE)
@@ -2995,7 +3151,7 @@ UMSTXTConv::encode (const UCS2 *in)
 	append (ucs[1]);
 	ucsr[0] = (endian==UCS2BE) ?  0 : '\r';
 	ucsr[1] = (endian==UCS2BE) ?  '\r' : 0;
-	for (i=0; in[i]!=0; i++)
+	for (i=0; i<inputSize; i++)
 	{
 		// 12 34 -> 12 34
 		if (endian==UCS2BE)
@@ -3025,10 +3181,8 @@ UMSTXTConv::encode (const UCS2 *in)
 		append (ucsr[1]);
 	}
 
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0);
-	append ((unsigned char) 0xff);
+	*retSize = size;
+	terminateStr();
 	return uchar;
 }
 
@@ -3037,17 +3191,16 @@ UMSTXTConv::encode (const UCS2 *in)
 // line  separator paragraph seperator and \r is 
 // translated.
 UCS2*
-UMSTXTConv::decode (const unsigned char *in)
+UMSTXTConv::decode (const unsigned char *in, int _size, int* retSize)
 {
 	int	i;
 	UCS2	ucs;
 
-	init (in);
+	init (in, _size);
 
 	// set some defualt state.
 	endian = UCS2LE;
-	for (i=0; !(in[i]==0 && in[i+1]==0);
-		i=i+2)
+	for (i=0; i<inputSize-1; i=i+2)
 	{
 		if (in[i] == (UNI_MAGIC>>8) 
 			&& in[i+1] == (UNI_MAGIC&0xff))
@@ -3082,6 +3235,8 @@ UMSTXTConv::decode (const unsigned char *in)
 		}
 		
 	}
-	append ((UCS2) 0);
+	if (inputSize % 2 != 0) append ((UCS2) '?');
+	terminateUCS2 ();
+	*retSize = size-1;
 	return ucs2;
 }
