@@ -1,9 +1,13 @@
-//  UFont.cpp version 1.1
+//  UFont.cpp version 1.5
 //  yudit package - Unicode Editor for the X Window System (and Linux) 
 //
 //  Author: gsinai@iname.com (Gaspar Sinai)
-//  GNU Copyright (C) 1997,1998  Gaspar Sinai
+//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
 // 
+//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
@@ -78,7 +82,8 @@ UFontCache::drawString (Display* dsp, Window window, GC gc,
 UFontX11Cache::UFontX11Cache (const char *name, Display* displayIn, 
 	const UFontMapStruct* fontMapIn, const int pixelIn, 
 	const UFont::UWeight weightIn, const UFont::USlant slantIn,
-	const UFont::USpacing spacingIn)
+	const UFont::USpacing spacingIn,
+	const char* addStyle, const char* avgWidth)
 {
 	font = 0;
 	cacheName = new char [strlen (name) +1];
@@ -86,7 +91,7 @@ UFontX11Cache::UFontX11Cache (const char *name, Display* displayIn,
 	strcpy (cacheName, name);
 
 	// try with this font 
-	fontName = getFontName (fontMapIn, pixelIn, weightIn, slantIn, spacingIn);
+	fontName = getFontName (fontMapIn, pixelIn, weightIn, slantIn, spacingIn, addStyle, avgWidth);
 	display = displayIn;
 	
 	// You have all the info to load the font here
@@ -174,7 +179,8 @@ char *
 UFontX11Cache::getFontName(
 	const UFontMapStruct* fontMapStruct, const int pixelIn, 
 	const UFont::UWeight weightIn, const UFont::USlant slantIn,
-	const UFont::USpacing spacingIn)
+	const UFont::USpacing spacingIn,
+	const char* addStyle, const char* avgWidth)
 {
 	char*		name;
 	/* The fontname is this */
@@ -183,13 +189,15 @@ UFontX11Cache::getFontName(
 		+ strlen (fontMapStruct->registry)
 		+ strlen (fontMapStruct->encoding) + 256];
 	CHECKNULL (name);
-	sprintf (name, "-%s-%s-%s-%s-*-*-%d-*-*-*-%s-*-%s-%s",
+	sprintf (name, "-%s-%s-%s-%s-*-%s-%d-*-*-*-%s-%s-%s-%s",
 		fontMapStruct->foundry, 
 		fontMapStruct->family,
 		weightMap[weightIn],
 		slantMap[slantIn],
+		addStyle,
 		pixelIn,
 		spacingMap[spacingIn],
+		avgWidth,
 		fontMapStruct->registry,
 		fontMapStruct->encoding);
 	return name;
@@ -205,6 +213,8 @@ UFont::UFont (UFontMap* fontMapIn)
 	slant = ANYSLANT;
 	spacing = ANYSPACING;
 	freeType = 0;
+	addStyle = "*";
+	avgWidth = "*";
 }
 
 UFont::UFont (UFreeType* freeTypeIn)
@@ -217,6 +227,8 @@ UFont::UFont (UFreeType* freeTypeIn)
 	slant = ANYSLANT;
 	spacing = ANYSPACING;
 	freeType = freeTypeIn;
+	addStyle = "*";
+	avgWidth = "*";
 }
 
 UFontCache *
@@ -254,6 +266,32 @@ UFont::setSpacing (const USpacing spacingIn)
 	spacing = spacingIn;
 }
 
+void
+UFont::setAddStyle (const char* addStyleIn)
+{
+	if (addStyleIn==0 || strcasecmp (addStyleIn, "Any") == 0)
+	{
+		addStyle = "*";
+	}
+	else
+	{
+		addStyle = addStyleIn;
+	}
+}
+
+void
+UFont::setAvgWidth (const char* avgWidthIn)
+{
+	if (avgWidthIn==0 || strcasecmp (avgWidthIn, "Any") == 0) 
+	{
+		avgWidth = "*";
+	}
+	else
+	{
+		avgWidth = avgWidthIn;
+	}
+}
+
 void
 UFont::setWeight (const char* weightIn)
 {
@@ -341,6 +379,11 @@ UFont::setSpacing (const char* spacingIn)
 		spacing = CONDENSED;
 		return;
 	}
+	if (strcasecmp (spacingIn, "CharCell") ==0)
+	{
+		spacing = CONDENSED;
+		return;
+	}
 	if (strcasecmp (spacingIn, "Proportional") ==0)
 	{
 		spacing = PROPORTIONAL;
@@ -387,6 +430,8 @@ UFontX11::UFontX11 (UFontX11 &fontCpy) : UFont (fontCpy.fontMap)
 	slant = fontCpy.slant;
 	pixel = fontCpy.pixel;
 	weight = fontCpy.weight;
+	addStyle = (const char*) fontCpy.addStyle;
+	avgWidth = (const char*) fontCpy.avgWidth;
 	size = 0;
 	fonts = 0;
 	fontIndex = 0;
@@ -448,6 +493,19 @@ UFontX11::setSpacing (const USpacing spacingIn)
 	spacing = spacingIn;
 }
 
+void
+UFontX11::setAddStyle (const char* addStyleIn)
+{
+	clear ();
+	UFont::setAddStyle (addStyleIn);
+}
+void
+UFontX11::setAvgWidth (const char* avgWidthIn)
+{
+	clear();
+	UFont::setAvgWidth (avgWidthIn);
+}
+
 void
 UFontX11::setWeight (const char* weightIn)
 {
@@ -506,6 +564,12 @@ UFontX11::getFont (const UCS2* text, const int from, int* to)
 	int				tryDescent[2];
 	char*				cacheName;
 
+	if (text != 0 && text[from]<0x20)
+	{
+		if (to!=0) *to=from+1;
+		return 0;
+	}
+
 	if (cache == 0)
 	{
 		cache = new UCache<UFontX11Cache*> (FONTCACHE_SIZE);
@@ -519,7 +583,11 @@ UFontX11::getFont (const UCS2* text, const int from, int* to)
 	}
 	if (fontChanged)
 	{
-		if (fontMap==0) return 0;
+		if (fontMap==0)
+		{
+			if (to!=0) *to=from+1;
+			return 0;
+		}
 		fontMap->init();
 		size = fontMap->size;
 		fonts = new UFontX11Cache* [size];
@@ -548,7 +616,10 @@ UFontX11::getFont (const UCS2* text, const int from, int* to)
 		// fill in the fonts
 		for (i=0; i<size; i++)
 		{
-			cacheName = getCacheName (fontMap->fonts[i]);
+			cacheName = getCacheName (fontMap->fonts[i],
+				(const char*) addStyle, 
+				(const char*) avgWidth);
+
 			fonts[i] = cache->getItem (cacheName);
 			if (fonts[i] != 0) 
 			{
@@ -556,8 +627,13 @@ UFontX11::getFont (const UCS2* text, const int from, int* to)
 			}
 			else
 			{
-				fonts[i] = new UFontX11Cache (cacheName, display, 
-					fontMap->fonts[i], pixel, weight, slant, spacing);
+				fonts[i] = new UFontX11Cache (cacheName, 
+					display, 
+					fontMap->fonts[i], pixel, 
+					weight, slant, spacing, 
+					(const char*) addStyle, 
+					(const char*) avgWidth);
+
 				(void) cache->addItem (cacheName, fonts[i]);
 				CHECKNULL (fonts[i]);
 			}
@@ -608,7 +684,7 @@ UFontX11::getFont (const UCS2* text, const int from, int* to)
 		fontChanged = 0;
 	}
 	// the guy just wanted to set the font bounding box.
-	if (text == 0 || text[0]==0) { return 0; }
+	if (text == 0) { return 0; }
 	i=from;
 
 	// get things with the same font.
@@ -703,7 +779,8 @@ UFontX11::getFontIndexCacheName ()
 }
 
 char *
-UFontX11::getCacheName (const UFontMapStruct*  fontMapStruct)
+UFontX11::getCacheName (const UFontMapStruct*  fontMapStruct,
+	const char* _addStyle, const char* _avgWidth)
 {
 	char*	name;
 	/* The fontname is this */
@@ -713,6 +790,8 @@ UFontX11::getCacheName (const UFontMapStruct*  fontMapStruct)
 		+ strlen (fontMapStruct->family) 
 		+ strlen (fontMapStruct->registry)
 		+ strlen (fontMapStruct->encoding)
+		+ strlen (_addStyle)
+		+ strlen (_avgWidth)
 		+ 256];
 	CHECKNULL (name);
 	sprintf (name, "%d %d %d %d ", weight, slant, pixel, spacing); 
@@ -725,6 +804,10 @@ UFontX11::getCacheName (const UFontMapStruct*  fontMapStruct)
 	strcat (name, fontMapStruct->registry);
 	strcat (name, " ");
 	strcat (name, fontMapStruct->encoding);
+	strcat (name, " ");
+	strcat (name, _addStyle);
+	strcat (name, " ");
+	strcat (name, _avgWidth);
 	return name;
 }
 
