@@ -1,9 +1,13 @@
-//  UInput.cpp version 1.1
+//  UInput.cpp version 1.5
 //  yudit package - Unicode Editor for the X Window System (and Linux) 
 //
 //  Author: gsinai@iname.com (Gaspar Sinai)
-//  GNU Copyright (C) 1997,1998  Gaspar Sinai
+//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
 // 
+//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
@@ -198,7 +202,7 @@ UInputSearch::addItem (const unsigned char* key, const UCS2* code)
 	int			index;
 	int			i;
 	int			keylen;
-	int			codelen;
+	int			codeLen = UCS2NullLen (code);
 	
 
 	if (arraySize <= size)
@@ -239,18 +243,17 @@ UInputSearch::addItem (const unsigned char* key, const UCS2* code)
 	}
 
 	keylen = strlen ((const char*)key);
-	codelen = UCS2Len (code);
-	
 	
 	myKey = new unsigned char[keylen+1];
 	CHECKNULL (myKey);
 
-	myCode = new UCS2 [codelen+1];
+	myCode = new UCS2 [codeLen+1];
 	CHECKNULL (myCode);
 
 	memcpy (myKey, key, (keylen +1) * sizeof (unsigned char));
 
-	memcpy (myCode, code, (codelen +1) * sizeof (UCS2));
+	memcpy (myCode, code, codeLen * sizeof (UCS2));
+	myCode[codeLen] = (UCS2) 0;
 
 	// move guys up
 	for (i=size; i>index; i--)
@@ -381,7 +384,7 @@ UInput::translate (const int asci)
 	{
 		copyStr[0] = (unsigned char) asci;
 		copyStr[1] = 0;
-		return UCS2Get (copyStr);
+		return UCS2Get (copyStr, 1);
 	}
 	if (asci==0) return 0;
 	stateString [stateSize++] = asci;
@@ -390,7 +393,7 @@ UInput::translate (const int asci)
 	if (stateSize >= UINPUT_STATE_SIZE-1)
 	{
 		cerr << "error: UInput stateSize overflow.\n";
-		ucs2= UCS2Get (stateString);
+		ucs2= UCS2Get (stateString, stateSize);
 		resetState ();
 		return ucs2;
 	}
@@ -417,7 +420,7 @@ UInput::translate (const int asci)
 			strcpy ((char*) copyStr,  
 				(const char*) &stateString [1]);
 			stateString [1] = 0;
-			tmpUcs2= UCS2Get (stateString);
+			tmpUcs2= UCS2Get (stateString, 1);
 
 			resetState ();
 			ucs2 = translateAdd (tmpUcs2, copyStr);
@@ -430,7 +433,7 @@ UInput::translate (const int asci)
 		ucs2Str[1] = 0;
 
 		// return the matched stuff.
-		tmpUcs2 = UCS2Dup (ucs2Str);
+		tmpUcs2 = UCS2Dup (ucs2Str, 1);
 
 		// move new string down.
 		strcpy ((char*) copyStr,  
@@ -447,7 +450,7 @@ UInput::translate (const int asci)
 	case MATCH:
 		if (inputCache->size==1)
 		{
-			ucs2= UCS2Dup (inputCache->search[0]->codes[index]);
+			ucs2= UCS2Dup (inputCache->search[0]->codes[index], UCS2NullLen(inputCache->search[0]->codes[index]));
 			resetState ();
 			return ucs2;
 		}
@@ -481,7 +484,7 @@ UInput::translate (const int asci)
 
 			ucs2Str[0] = accumulated[0][0] + accumulated[0][1];
 			ucs2Str[1] = 0;
-			ucs2 = UCS2Dup (ucs2Str);
+			ucs2 = UCS2Dup (ucs2Str, 1);
 
 			gccBug = accumulated[1][0];
 			accumulated[0][1] = 0;
@@ -503,7 +506,8 @@ UInput::translate (const int asci)
 		//
 		if (inputCache->size==1)
 		{
-			tmpUcs2= UCS2Dup (inputCache->search[0]->codes[index]);
+			tmpUcs2= UCS2Dup (inputCache->search[0]->codes[index],
+				UCS2NullLen (inputCache->search[0]->codes[index]));
 			strcpy ((char*) copyStr,  (const char*) &stateString [ 
 				strlen ((const char*) 
 				inputCache->search[0]->keys[index])] );
@@ -535,7 +539,7 @@ UInput::translate (const int asci)
 			ucs2Str[0] = accumulated[0][0] + accumulated[0][1];
 			ucs2Str[1] = 0;
 			resetState ();
-			tmpUcs2 = UCS2Dup (ucs2Str);
+			tmpUcs2 = UCS2Dup (ucs2Str, 1);
 			ucs2 = translateAdd (tmpUcs2, copyStr);
 			return ucs2;
 		}
@@ -564,7 +568,7 @@ UInput::translate (const int asci)
 		ucs2Str[1] = 0;
 		resetState ();
 
-		tmpUcs2 = UCS2Dup (ucs2Str);
+		tmpUcs2 = UCS2Dup (ucs2Str, 1);
 		ucs2 = translateAdd (tmpUcs2, copyStr);
 		return ucs2;
 
@@ -589,7 +593,10 @@ UInput::translateAdd (UCS2* ucs2_, const unsigned char* str)
 		{
 			if (tmpUcs2!=0)
 			{
-				ucs2 = UCS2Concat (tmpUcs2, ucs2);
+				ucs2 = UCS2Concat (tmpUcs2, 
+					UCS2NullLen (tmpUcs2),
+					ucs2,
+					UCS2NullLen (ucs2));
 				delete tmpUcs2;
 			}
 			tmpUcs2 = ucs2;
