@@ -1,9 +1,13 @@
-//  UPrint.cpp version 1.1
+//  UPrint.cpp version 1.5
 //  yudit package - Unicode Editor for the X Window System (and Linux) 
 //
 //  Author: gsinai@iname.com (Gaspar Sinai)
-//  GNU Copyright (C) 1997,1998  Gaspar Sinai
+//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
 // 
+//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
@@ -64,9 +68,9 @@ static double f2dot14 (short x);
 static void  printFloat (ostream* ostr, double f);
 static void  printUnic (ostream* ostr, int u);
 
-static char* psCreator = "yudit 1.1 GNU (C) Gaspar Sinai";
+static char* psCreator = "yudit 1.5 GNU (C) Gaspar Sinai";
 static UCS2 headerStr[] = {'0', '1', '2', '3', '4', '5', '6', '7', 
-   '8', '9', 'o', 'f', '.', '/', ' ', 0};
+   '8', '9', 'o', 'f', '.', '/', ' ', '-', ':', 0};
 
 typedef struct _TableMap
 {
@@ -324,8 +328,7 @@ UPrint::~UPrint ()
 }
 
 UPrint::UStatus
-UPrint::print (ostream* printerIn, const UCS2* const linesIn[], 
-	int countIn, const UCS2* headerIn)
+UPrint::print (ostream* printerIn, const UCS2* const linesIn[], const int lineSizes[], int countIn, const UCS2* headerIn, const int headerL)
 {
 	int		i;
 	int		j;
@@ -351,6 +354,7 @@ UPrint::print (ostream* printerIn, const UCS2* const linesIn[],
 	// get the character map.
 	clear ();
 	headerString = headerIn;
+	headerLength = headerL;
 	if (stat ((const char*) fontFile, &statbuf) == -1)
 	{
 		errors = "error: can not access font '";
@@ -370,7 +374,7 @@ UPrint::print (ostream* printerIn, const UCS2* const linesIn[],
 		errors += "'.";
 		return ERROR;
 	}
-	fileBuffer = mmap (0, fileBufferLen,  PROT_READ, MAP_PRIVATE, fd, 0);
+	fileBuffer = (char*) mmap (0, fileBufferLen,  PROT_READ, MAP_PRIVATE, fd, 0);
 	if (fileBuffer==0)
 	{
 		errors = "error: can not mmap font  '";
@@ -469,13 +473,13 @@ UPrint::print (ostream* printerIn, const UCS2* const linesIn[],
 	// Find out what fonts are needed
 	for (i=0; i<countIn; i++)
 	{
-		addMap (linesIn[i]);
+		addMap (linesIn[i], lineSizes[i]);
 	}
 	// Add header characters if necessary
 	if (headerHeight > 0.0)
 	{
-		addMap (headerStr);
-		if (headerIn!=0) addMap (headerIn);
+		addMap (headerStr, sizeof (headerStr) / sizeof (UCS2));
+		if (headerIn!=0) addMap (headerIn, headerL);
 	}
 
 	if (buildCmap()==ERROR)
@@ -551,7 +555,7 @@ UPrint::print (ostream* printerIn, const UCS2* const linesIn[],
 			pageCount++;
 			newPage = 0;
 		}
-		for (j=0; linesIn[i][j] != 0; j++)
+		for (j=0; j < lineSizes [i]; j++)
 		{
 			if (newPage!=0)
 			{
@@ -584,7 +588,7 @@ UPrint::print (ostream* printerIn, const UCS2* const linesIn[],
 				// print here.
 				cursorX += lastWidth;
 
-				if (linesIn[i][j+1] != 0)
+				if (j+1 < lineSizes[i])
 				{
 					cursorX = marginWidth;
 					cursorY-=lineHeight;
@@ -626,7 +630,7 @@ UPrint::print (ostream* printerIn, const UCS2* const linesIn[],
 			printChar (0, 0, NEWPAGE);
 			newPage = 0;
 		}
-		for (j=0; linesIn[i][j] != 0; j++)
+		for (j=0; j < lineSizes[i]; j++)
 		{
 			if (newPage!=0)
 			{
@@ -658,7 +662,7 @@ UPrint::print (ostream* printerIn, const UCS2* const linesIn[],
 				lastWidth = printChar(linesIn[i][j], xPrintMap,
 				     FORCE);
 
-				if (linesIn[i][j+1] != 0)
+				if (j+1 < lineSizes[i])
 				{
 					wrappedCount++;
 					cursorX = marginWidth;
@@ -719,12 +723,12 @@ UPrint::print (ostream* printerIn, const UCS2* const linesIn[],
 }
 
 void
-UPrint::addMap (const UCS2* lineIn)
+UPrint::addMap (const UCS2* lineIn, const int _size)
 {
 	int		i;
 	unsigned short	indexR;
 	unsigned short	indexC;
-	for (i=0; lineIn[i] != 0; i++)
+	for (i=0; i<_size; i++)
 	{
 		indexR = lineIn[i] >> 8;
 		indexC = lineIn[i] & 0xff;
@@ -749,6 +753,9 @@ UPrint::doHeader ()
 	double		headerScale;
 	double 		baseLine;
 	double 		headerLine;
+	int		dateLen;
+	int		pageLen;
+	
 	UCS2		ucs2;
 	time_t		now;
 	struct tm*	localTime;
@@ -760,6 +767,8 @@ UPrint::doHeader ()
 	// Do the header numbering.
 	sprintf (pageStr, "%d of %d", pageIndex, pageCount);
 	length = 0.0;
+	dateLen = 0;
+	pageLen = 0;
 	for (i=0; pageStr[i] != 0; i++)
 	{
 		length += printMap[0][pageStr[i]]->width;
@@ -778,23 +787,30 @@ UPrint::doHeader ()
 	printChar (0, 0, MOVE);
 	for (i=0; pageStr[i] != 0; i++)
 	{
+		pageLen += printMap[0][pageStr[i]]->width;
 		printChar (pageStr[i], printMap[0][pageStr[i]], FORCE);
 	}
+	pageLen += printMap[0][' ']->width;
 	printChar (0, 0, FINISH);
 
 	time(&now);
 	localTime = localtime (&now);
+
 	if (localTime!=0)
 	{
 		memset (pageStr, 0, sizeof (pageStr));
-		strftime (pageStr, sizeof (pageStr)-1, "%m/%d/%Y", localTime);
+		// Let's stick to international format
+		//strftime (pageStr, sizeof (pageStr)-1, "%m/%d/%Y", localTime);
+		strftime (pageStr, sizeof (pageStr) - 1, "%Y-%m-%d %H:%M:%S", localTime);
 		cursorX = marginWidth;
 		cursorY = baseLine; 
 		printChar (0, 0, MOVE);
-		for (i=0; pageStr[i] != 0; i++)
+		for (i=0; pageStr[i] != 0 && i < sizeof (pageStr); i++)
 		{
+			dateLen += printMap[0][pageStr[i]]->width;
 			printChar (pageStr[i], printMap[0][pageStr[i]], FORCE);
 		}
+		dateLen += printMap[0][' ']->width;
 		printChar (0, 0, FINISH);
 	}
 	// Draw line
@@ -810,17 +826,27 @@ UPrint::doHeader ()
 		return;
 	}
 	length =  2 * printMap[0]['.']->width;
-	for (i=UCS2Len(headerString)-1; i >= 0; i--)
+	for (i=headerLength-1; i >= 0; i--)
 	{
 		ucs2 = headerString[i];
 		length += printMap[ucs2>>8][ucs2&0xff]->width;
-		if (length*headerScale >= mediaWidth/2) break;
+		//if (length*headerScale >= mediaWidth/2) break;
+		if ((length+dateLen+2*pageLen)*headerScale >= mediaWidth - 2*marginWidth) break;
 	}
 	// lets reuse this 
 	length = length * headerScale;
 	// print it out
 	printChar (0, 0, FINISH);
-	cursorX = mediaWidth/2 - length/2;
+	
+	if (dateLen*headerScale + marginWidth < mediaWidth/2 - length/2)
+	{
+		cursorX = mediaWidth/2 - length/2;
+	}
+	else
+	{
+		cursorX = dateLen*headerScale + marginWidth;
+	}
+
 	cursorY = baseLine; 
 	printChar (0, 0, MOVE);
 	if (i>0)
@@ -832,8 +858,9 @@ UPrint::doHeader ()
 	{
 		i=0;
 	}
-	while ((ucs2 = headerString[i])!=0)
+	while (i<headerLength)
 	{
+		ucs2 = headerString[i];
 		printChar (ucs2, printMap[ucs2>>8][ucs2&0xff], FORCE);
 		i++;
 	}
