@@ -1,9 +1,13 @@
-//  UWidget.cpp version 1.1
+//  UWidget.cpp version 1.5
 //  yudit package - Unicode Editor for the X Window System (and Linux) 
 //
 //  Author: gsinai@iname.com (Gaspar Sinai)
-//  GNU Copyright (C) 1997,1998  Gaspar Sinai
+//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
 // 
+//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
@@ -56,9 +60,11 @@ UWidget::~UWidget ()
 	if (pixmapFonts) delete pixmapFonts;
 	if (uDisplay != 0)
 	{
-		XFreeGC (uDisplay, ugc);
-		XFreeGC (uDisplay, ucgc);
-		XFreeGC (uDisplay, urgc);
+		// GC should go away with window.
+		// BUG?
+		//XFreeGC (uDisplay, ugc);
+		//XFreeGC (uDisplay, ucgc);
+		//XFreeGC (uDisplay, urgc);
 	}
 	if (uCursor!=0) delete uCursor;
 	if (char2BLen!=0) delete char2BBuf;
@@ -81,6 +87,7 @@ UWidget::UWidget (const char* nameIn,
 	uTextSize = 0;
 	uTextSizes = 0;
 	uTextTabSizes = 0;
+	uTextCtrlSizes = 0;
 	uTextBuffer = 0;
 	pixmapFonts = 0;
 	uX = 0;
@@ -134,6 +141,7 @@ UWidget::UWidget (const char* nameIn, Display* displayIn, Window windowIn,
 	uTextSize = 0;
 	uTextSizes = 0;
 	uTextTabSizes = 0;
+	uTextCtrlSizes = 0;
 	uTextBuffer = 0;
 	uX = 0;
 	uY = 0;
@@ -317,6 +325,7 @@ UWidget::uProcMousePressEvent (const UMouseButton but,
 	const int xIn, const int yIn)
 {
 	UPoint		textXY;
+	int		cliplen;
 	unsigned char*	chars;
 
 	if (underLine[0] != underLine[1])
@@ -359,8 +368,8 @@ UWidget::uProcMousePressEvent (const UMouseButton but,
 			clearSelection ();
 			uSetCursorXY (textXY.posX, textXY.posY);
 		}
-		chars = uClipboardGet();
-		uInsertText (chars, 1);
+		chars = uClipboardGet(&cliplen);
+		uInsertText (chars, cliplen, 1, 0);
 		uSetFullSize ();
 		uSetCursorViewable();
 		delete chars;
@@ -958,12 +967,9 @@ UWidget::uProcKeyPressEvent (const UKey key, const int asci)
 			underLine[0] = underLine[1];
 		}
 		clearSelection();
-		chars[0] = (unsigned char) '\n';
-		chars[1] = 0;
-		saveConverter = uString;
-		uString = 0;
-		uInsertText (chars, 1);
-		uString = saveConverter;
+		ucsTwo[0] = (unsigned char) '\n';
+		ucsTwo[1] = 0;
+		uInsertUCS2 (ucsTwo, 1);
 		uSetFullSize ();
 		uSetCursorViewable();
 		return;
@@ -982,7 +988,7 @@ UWidget::uProcKeyPressEvent (const UKey key, const int asci)
 		clearSelection();
 		ucsTwo[0] = U_LINE_SEP;
 		ucsTwo[1] = 0;
-		uInsertUCS2 (ucsTwo);
+		uInsertUCS2 (ucsTwo, 1);
 		uSetFullSize ();
 		uSetCursorViewable();
 		break;
@@ -1001,7 +1007,7 @@ UWidget::uProcKeyPressEvent (const UKey key, const int asci)
 		clearSelection();
 		ucsTwo[0] = U_PARA_SEP;
 		ucsTwo[1] = 0;
-		uInsertUCS2 (ucsTwo);
+		uInsertUCS2 (ucsTwo, 1);
 		uSetFullSize ();
 		uSetCursorViewable();
 		break;
@@ -1027,7 +1033,25 @@ UWidget::uProcKeyPressEvent (const UKey key, const int asci)
 		}
 	case UK_ASCII:
 		if (editable!=EDITABLE) return;
-		if (asci == 0) return;
+		if (asci == 0)
+		{
+			if (underLine[0] != underLine[1])
+			{
+				// Finish it with space
+				uProcKeyPressEvent (UK_ASCII, '\001');
+				uProcKeyPressEvent (UK_BACKSPACE, 0);
+				uInput->resetState ();
+				hideUnderLine();
+				underLine[0] = underLine[1];
+			}
+			clearSelection();
+			ucsTwo[0] = 0;
+			ucsTwo[1] = 0;
+			uInsertUCS2 (ucsTwo, 1);
+			uSetFullSize ();
+			uSetCursorViewable();
+			return;
+		}
 		if (asci == 27 && uXInput!=0)
 		{
 			if (uXInputActive)
@@ -1058,14 +1082,14 @@ UWidget::uProcKeyPressEvent (const UKey key, const int asci)
 				uDeleteText (underLine[0], underLine[1]);
 			}
 			underLine[1] = underLine[0];
-			uInsertUCS2 (ucs2);
+			uInsertUCS2 (ucs2, UCS2NullLen(ucs2));
 			delete ucs2;
 			if (uInput->getStateSize()>0)
 			{
 				underLine[0] = uCursorBox.at;
 				underLine[1] = uCursorBox.at;
 				state = uInput->getUCS2 ();
-				uInsertUCS2 (state);
+				uInsertUCS2 (state, UCS2NullLen(state));
 				underLine[1] = uCursorBox.at;
 				drawUnderLine ();
 				delete state;
@@ -1080,13 +1104,13 @@ UWidget::uProcKeyPressEvent (const UKey key, const int asci)
 		{
 			underLine[0] = uCursorBox.at;
 			underLine[1] = uCursorBox.at;
-			uInsertUCS2 (ucsTwo);
+			uInsertUCS2 (ucsTwo, 1);
 			underLine[1] = uCursorBox.at;
 		}
 		else
 		{
 			underLine[1] = uCursorBox.at;
-			uInsertUCS2 (ucsTwo);
+			uInsertUCS2 (ucsTwo, 1);
 			underLine[1] = uCursorBox.at;
 		}
 		drawUnderLine ();
@@ -1319,6 +1343,7 @@ UWidget::drawSegments (UPoint& from, const UPoint& to, const UDrawType type)
 			    // both points after 
 			    || reversePoints[reverseP] >= segmentTo)
 			{
+
 				lastWidth = drawSegment (from, 
 					segmentTo, text, fontCache, 
 					S_NORMAL);
@@ -2025,6 +2050,8 @@ UWidget::uSetFont  (const char* fontIn)
 	UFont::USlant	xSlant;
 	UFont::USpacing	xSpacing;
 	UFreeTypeFace*	xFace;
+	AString		avgWidth;
+	AString		addStyle;
 
 	if (uDisplay==0) return ERROR;
 
@@ -2052,6 +2079,8 @@ UWidget::uSetFont  (const char* fontIn)
 		xWeight = uFont->getWeight();
 		xSlant = uFont->getSlant();
 		xSpacing = uFont->getSpacing();
+		avgWidth =  uFont->getAvgWidth();
+		addStyle =  uFont->getAddStyle();
 		if (fMap==0)
 		{
 			xFace=fType->getFreeTypeFace (
@@ -2085,6 +2114,8 @@ UWidget::uSetFont  (const char* fontIn)
 	uFont->setWeight(xWeight);
 	uFont->setSlant(xSlant);
 	uFont->setSpacing(xSpacing);
+	uFont->setAddStyle((const char*) addStyle);
+	uFont->setAvgWidth((const char*) avgWidth);
 	if (uXInputActive)
 	{
 		uXInput->uChangeAttributes (UXInput::UFONT);
@@ -2153,13 +2184,41 @@ UWidget::uSetFontSpacing (const char* spacing)
 	return OK;
 }
 
+UWidget::UStatus
+UWidget::uSetFontAddStyle (const char* addStyle)
+{
+	if (uFont==0) return ERROR;
+	if (underLine[0] != underLine[1])
+	{
+		hideUnderLine();
+		underLine[0]= underLine[1];
+	}
+	clearSelection();
+	if (pixmapFonts) pixmapFonts->clear();
+	uFont->setAddStyle (addStyle);
+	return OK;
+}
+
+UWidget::UStatus
+UWidget::uSetFontAvgWidth (const char* avgWidth)
+{
+	if (uFont==0) return ERROR;
+	if (underLine[0] != underLine[1])
+	{
+		hideUnderLine();
+		underLine[0]= underLine[1];
+	}
+	clearSelection();
+	if (pixmapFonts) pixmapFonts->clear();
+	uFont->setAvgWidth (avgWidth);
+	return OK;
+}
+
 UString::UStatus
 UWidget::uSetConverter  (const char* nameIn)
 {
-	static unsigned char nullTerm[] = { 0, 0, 0, 0xff };
 	if (clipboardActive!=0)
 	{
-		uClipboardPut (nullTerm);
 		uClipboardClear ();
 	}
 	if (uString != 0) delete uString;
@@ -2256,9 +2315,11 @@ UWidget::uClear  ()
 	if (uTextBuffer) delete uTextBuffer;
 	if (uTextSizes) delete uTextSizes;
 	if (uTextTabSizes) delete uTextTabSizes;
+	if (uTextCtrlSizes) delete uTextCtrlSizes;
 	uTextBuffer = 0;
 	uTextSizes = 0;
 	uTextTabSizes = 0;
+	uTextCtrlSizes = 0;
 	uTextBufferSize = 0;
 	uSetCursorXY (0, 0);
 	return OK;
@@ -2276,6 +2337,7 @@ UWidget::uHandleEvent (XEvent *event)
 	int			activeFlag;
 	UKey			ukey;
 	unsigned char*		chars;
+	int			cliplen;
 
 	switch (event->type)
 	{
@@ -2343,8 +2405,8 @@ UWidget::uHandleEvent (XEvent *event)
 			{
 				return;
 			}
-			chars = uClipboardGet();
-			uInsertText (chars, 1);
+			chars = uClipboardGet(&cliplen);
+			uInsertText (chars, cliplen, 1, 0);
 			uSetFullSize ();
 			uSetCursorViewable();
 			delete chars;
@@ -2712,7 +2774,7 @@ UWidget::uSetFullSize()
 	newHeight = uTextSize * uFont->fontHeight + uBorderSize * 2;
 	for (i=0; i<uTextSize; i++)
 	{
-		newSize = uFont->fontWidth * (uTextSizes[i] 
+		newSize = uFont->fontWidth * (uTextSizes[i] + uTextCtrlSizes[i]
 			   + (tabSize-1) * uTextTabSizes[i]);
 		if (maxWidth < newSize) maxWidth = newSize;
 	}
@@ -3006,11 +3068,13 @@ UWidget::uInsertLineUCS2 (UPoint& currentP, const UCS2 *text,
 	UCS2**		newBuffer;
 	int*		newSizes;
 	int*		newTabSizes;
+	int*		newCtrlSizes;
 	UCS2*		newTextBuffer;
 	int		newSize;
 	int		len;
 	int		i;
 	int		newTabs;
+	int		newCtrls;
 	int		endPart;
 
 	clearSelection ();
@@ -3026,25 +3090,36 @@ UWidget::uInsertLineUCS2 (UPoint& currentP, const UCS2 *text,
 		CHECKNULL (newSizes);
 		newTabSizes = new int [newSize];
 		CHECKNULL (newTabSizes);
+		newCtrlSizes = new int [newSize];
+		CHECKNULL (newCtrlSizes);
 		if (uTextBufferSize != 0)
 		{
 			memcpy (newBuffer, uTextBuffer, 
 				uTextSize * sizeof (UCS2 *));
 			delete uTextBuffer;
+
 			memcpy (newSizes, uTextSizes,
 				uTextSize * sizeof (int));
+			delete uTextSizes;
+
 			memcpy (newTabSizes, uTextTabSizes,
 				uTextSize * sizeof (int));
-			delete uTextSizes;
+			delete uTextTabSizes;
+
+			memcpy (newCtrlSizes, uTextCtrlSizes,
+				uTextSize * sizeof (int));
+			delete uTextCtrlSizes;
 		}
 		uTextBuffer = newBuffer;
 		uTextSizes = newSizes;
 		uTextTabSizes = newTabSizes;
+		uTextCtrlSizes = newCtrlSizes;
 		uTextBufferSize = newSize;
 	}
 	uTextBuffer[uTextSize] = 0;
 	uTextSizes[uTextSize] = 0;
 	uTextTabSizes[uTextSize] = 0;
+	uTextCtrlSizes[uTextSize] = 0;
 
 	// open new line
 	if (newLine!=0)
@@ -3056,6 +3131,7 @@ UWidget::uInsertLineUCS2 (UPoint& currentP, const UCS2 *text,
 			uTextBuffer[i] = uTextBuffer[i-1];
 			uTextSizes[i] = uTextSizes[i-1];
 			uTextTabSizes[i] = uTextTabSizes[i-1];
+			uTextCtrlSizes[i] = uTextCtrlSizes[i-1];
 		}
 		
 		endPart = 0;
@@ -3067,6 +3143,7 @@ UWidget::uInsertLineUCS2 (UPoint& currentP, const UCS2 *text,
 		uTextBuffer[currentP.posY] = new UCS2 [to-from + 1 + endPart];
 		CHECKNULL (uTextBuffer[currentP.posY]);
 		newTabs = 0;
+		newCtrls = 0;
 		if (endPart>0)
 		{
 			for (i=0; i<endPart; i++)
@@ -3074,21 +3151,37 @@ UWidget::uInsertLineUCS2 (UPoint& currentP, const UCS2 *text,
 				uTextBuffer[currentP.posY][i+to-from] =
 					uTextBuffer[currentP.posY-1][currentP.posX+i];
 				if (uTextBuffer[currentP.posY][i+to-from] 
-					== (UCS2) '\t') newTabs++;
+					== (UCS2) '\t')
+				{
+					 newTabs++;
+				}
+				else if (uTextBuffer[currentP.posY][i+to-from] < ' ')
+				{
+					 newCtrls++;
+				}
 			}
 			// should I free it up ?
 			uTextBuffer[currentP.posY-1][currentP.posX] = 0;
 			uTextTabSizes[currentP.posY-1] -= newTabs;
+			uTextCtrlSizes[currentP.posY-1] -= newCtrls;
 			uTextSizes[currentP.posY-1] -=  endPart;
 		}
 		for (i=from; i<to; i++)
 		{
 			uTextBuffer[currentP.posY][i-from] = text[i];
-			if (text[i] == (UCS2) '\t') newTabs++;
+			if (text[i] == (UCS2) '\t')
+			{
+				 newTabs++;
+			}
+			else if (text[i] < ' ')
+			{
+				 newCtrls++;
+			}
 		}
 		uTextBuffer[currentP.posY][to-from+endPart] = 0;
 		uTextSizes[currentP.posY] = to-from+endPart;
 		uTextTabSizes[currentP.posY] = newTabs;
+		uTextCtrlSizes[currentP.posY] = newCtrls;
 		uTextSize++;
 		currentP.posX=to-from;
 		return;
@@ -3110,11 +3203,19 @@ UWidget::uInsertLineUCS2 (UPoint& currentP, const UCS2 *text,
 			currentP.posX * sizeof (UCS2));
 	}
 	newTabs = 0;
+	newCtrls = 0;
 	// Text
 	for (i=from; i<to; i++)
 	{
 		newTextBuffer[i-from+currentP.posX] = text[i];
-		if (text[i] == (UCS2) '\t') newTabs++;
+		if (text[i] == (UCS2) '\t')
+		{
+			newTabs++;
+		}
+		else if (text[i] < ' ')
+		{
+			newCtrls++;
+		}
 	}
 	// After text
 	if (currentP.posX != uTextSizes[currentP.posY])
@@ -3131,6 +3232,7 @@ UWidget::uInsertLineUCS2 (UPoint& currentP, const UCS2 *text,
 	if (uTextBuffer[currentP.posY]!=0) delete uTextBuffer[currentP.posY];
 	uTextBuffer[currentP.posY] = newTextBuffer;
 	uTextTabSizes[currentP.posY] += newTabs;
+	uTextCtrlSizes[currentP.posY] += newCtrls;
 	currentP.posX += to-from;
 	if (uTextSize==0) uTextSize=1;
 	return;
@@ -3230,12 +3332,18 @@ UWidget::endSelection (const int coordXIn, const int coordYIn,
 	int			col;
 	int			count;
 	unsigned char*		bytes;
+	int			byteLen;
 
 	if (startDrag<=0)
 	{
 		startDrag = 0;
 		return;
 	}
+	if (uTextSizes == 0)
+	{
+		startDrag = 0;
+		return;
+	}
 	if (reversePoints[0].posX <0)
 	{
 		clearSelection();
@@ -3291,7 +3399,7 @@ UWidget::endSelection (const int coordXIn, const int coordYIn,
 			col++)
 	
 		{
-			if (col > uTextSizes[row] || col < 0) break;
+			if (uTextSize<= row || col > uTextSizes[row] || col < 0) break;
 			ucs2[count++] = uTextBuffer[row][col];
 		}
 		if (count <=0 || (ucs2[count-1] != U_PARA_SEP  &&
@@ -3305,10 +3413,11 @@ UWidget::endSelection (const int coordXIn, const int coordYIn,
 		count--;
 	}
 	ucs2[count] = 0;
-	uString->putUString (ucs2);
-	bytes = uGetMarkedText();
+	uString->putUString (ucs2, count);
+	bytes = uGetMarkedText(&byteLen);
 	clipboardActive=1;
-	uClipboardPut (bytes);
+	uClipboardPut (bytes, byteLen);
+
 	delete bytes;
 	delete ucs2;
 }
@@ -3332,11 +3441,11 @@ UWidget::clearSelection ()
 }
 
 unsigned char*
-UWidget::uGetMarkedText()
+UWidget::uGetMarkedText(int* bytelen)
 {
 	unsigned char* bytes;
 	if (uString==0) return 0;
-	bytes =  uString->getString ();
+	bytes =  uString->getString (bytelen);
 	if (bytes == 0)
 	{
 		cerr << "error: count not encode selection";
@@ -3346,16 +3455,16 @@ UWidget::uGetMarkedText()
 
 // You need to free this up
 unsigned char*
-UWidget::uGetLine (const int lineNo)
+UWidget::uGetLine (const int lineNo, int* clen)
 {
 	if (lineNo< 0 || lineNo>= uTextSize) return 0;
 	if (uString ==0) return 0;
-	uString->putUString ((UCS2*) uTextBuffer[lineNo]);
-	return  uString->getString ();
+	uString->putUString ((UCS2*) uTextBuffer[lineNo], uTextSizes[lineNo]);
+	return  uString->getString (clen);
 }
 
 unsigned char*
-UWidget::uGetBuffer ()
+UWidget::uGetBuffer (int* buflen)
 {
 	int		i;
 	int		len;
@@ -3393,15 +3502,15 @@ UWidget::uGetBuffer ()
 		if (retVle[index-1]=='\n') index--;
 	}
 	retVle[index]=0;
-	uString->putUString ((UCS2*) retVle);
+	uString->putUString ((UCS2*) retVle, index);
 	delete (retVle);
-	retChar =  uString->getString ();
-	uString->putUString ((UCS2*) 0);
+	retChar =  uString->getString (buflen);
+	uString->putUString ((UCS2*) 0, 0);
 	return retChar;
 }
 
 void
-UWidget::uAppendText (const unsigned char *text,
+UWidget::uAppendText (const unsigned char *text, const int _length,
 	const int moveCursor, const int newLine)
 {
 	UCursorBox	saveBox;
@@ -3427,7 +3536,7 @@ UWidget::uAppendText (const unsigned char *text,
 				uTextSize-1);
 		}
 	}
-	uInsertText (text, moveCursor, newLine);
+	uInsertText (text, _length, moveCursor, newLine);
 	if (moveCursor==0) uCursorBox = saveBox;
 }
 
@@ -3439,6 +3548,7 @@ UWidget::uDeleteText (const UPoint& from, const UPoint& to)
 	UCS2*		newText;
 	int		newTextSize;
 	int		newTabs;
+	int		newCtrls;
 	UPoint		startP;
 	UPoint		endP;
 	UCursorState	saveState;
@@ -3458,17 +3568,32 @@ UWidget::uDeleteText (const UPoint& from, const UPoint& to)
 	newTextSize = from.posX + uTextSizes[to.posY] - to.posX;
 	newText = new UCS2 [newTextSize+1];
 	newTabs = 0;
+	newCtrls = 0;
 	count = 0;
 	for (i=0; i<from.posX; i++)
 	{
 		newText[count] = uTextBuffer[from.posY][i];
-		if (newText[count] == (UCS2) '\t') newTabs++;
+		if (newText[count] == (UCS2) '\t')
+		{
+			newTabs++;
+		}
+		else if (newText[count] < ' ')
+		{
+			newCtrls++;
+		}
 		count++;
 	}
 	for (i=to.posX; i<uTextSizes[to.posY]; i++)
 	{
 		newText[count] = uTextBuffer[to.posY][i];
-		if (newText[count] == (UCS2) '\t') newTabs++;
+		if (newText[count] == (UCS2) '\t')
+		{
+			newTabs++;
+		}
+		else if (newText[count] < ' ')
+		{
+			newCtrls++;
+		}
 		count++;
 	}
 	newText[count] = 0;
@@ -3480,6 +3605,7 @@ UWidget::uDeleteText (const UPoint& from, const UPoint& to)
 	uTextBuffer[from.posY] = newText;
 	uTextSizes[from.posY] = newTextSize;
 	uTextTabSizes[from.posY] = newTabs;
+	uTextCtrlSizes[from.posY] = newCtrls;
 
 	// move the guys from up to down
 	if (from.posY != to.posY)
@@ -3489,6 +3615,7 @@ UWidget::uDeleteText (const UPoint& from, const UPoint& to)
 			uTextBuffer[from.posY+i] = uTextBuffer[i+to.posY];
 			uTextSizes[from.posY+i] = uTextSizes[i+to.posY];
 			uTextTabSizes[from.posY+i] = uTextTabSizes[i+to.posY];
+			uTextCtrlSizes[from.posY+i] = uTextCtrlSizes[i+to.posY];
 		}
 		uTextSize -= to.posY-from.posY;
 	}
@@ -3549,12 +3676,13 @@ UWidget::uDeleteText (const UPoint& from, const UPoint& to)
 
 // middle button
 void 
-UWidget::uInsertText (const unsigned char *text, const int moveCursor, 
-	const int newLineIn)
+UWidget::uInsertText (const unsigned char *text, 
+	const int _length, const int moveCursor, const int newLineIn)
 {
 	int		i;
-	const UCS2*	converted;
+	UCS2*		converted;
 	UCS2*		clean;
+	int		slen;
 
 	if (text == 0)
 	{
@@ -3564,9 +3692,10 @@ UWidget::uInsertText (const unsigned char *text, const int moveCursor,
 	}
 	if (uString != 0)
 	{
-		uString->putString (text);
-		converted = (const UCS2*) *uString;
-		uInsertUCS2 (converted, moveCursor, newLineIn);
+		uString->putString (text, _length);
+		converted = uString->getUString (&slen);
+		uInsertUCS2 (converted, slen, moveCursor, newLineIn);
+		delete converted;
 	}
 	else
 	{
@@ -3577,17 +3706,16 @@ UWidget::uInsertText (const unsigned char *text, const int moveCursor,
 			clean[i] = (UCS2) text[i];
 		}
 		clean[i] = 0;
-		uInsertUCS2 (clean, moveCursor, newLineIn);
+		uInsertUCS2 (clean, i, moveCursor, newLineIn);
 		delete clean;
 	}
 }
 
 void 
-UWidget::uInsertUCS2 (const UCS2 *converted, const int moveCursor, 
-	const int newLineIn)
+UWidget::uInsertUCS2 (const UCS2 *converted, const int len, 
+	const int moveCursor, const int newLineIn)
 {
 	int		i;
-	int		len;
 	int		from;
 	int		to;
 	UPoint		startP;
@@ -3598,9 +3726,7 @@ UWidget::uInsertUCS2 (const UCS2 *converted, const int moveCursor,
 	int		incl;
 
 	if (editable != EDITABLE) return;
-
-	len = UCS2Len (converted);
-	if (len==0) return;
+	if (len == 0) return;
 
 	startP = uCursorBox.at;
 	currentP = startP;
@@ -3704,19 +3830,19 @@ UWidget::uClipboardClear()
 {
 	clipboardActive=0;
 	clearSelection();
-	if (uString!=0) uString->putUString ((UCS2*) 0);
+	if (uString!=0) uString->putUString ((UCS2*) 0, 0);
 }
 
 // Put text to clipboard 
 void
-UWidget::uClipboardPut (const unsigned char *text)
+UWidget::uClipboardPut (const unsigned char *text, const int len )
 {
 	clipboardActive=1;
 }
 
 // get text from clipboard 
 unsigned char*	
-UWidget::uClipboardGet ()
+UWidget::uClipboardGet (int *len)
 {
 	return 0;
 }
@@ -3787,10 +3913,9 @@ UWidget::uDeleteSelection()
 }
 
 int
-UWidget::uFindUCS2 (const UCS2* text, int fromBegin)
+UWidget::uFindUCS2 (const UCS2* text, const int len, int fromBegin)
 {
 	int		i,j,k;
-	int		len;
 	UPoint		fromP;
 	UPoint		toP;
 	int		line;
@@ -3799,7 +3924,6 @@ UWidget::uFindUCS2 (const UCS2* text, int fromBegin)
 	int		toY;
 
 	if (text==0 || text[0] == 0) return -1;
-	len = UCS2Len (text);
 	if (fromBegin==0)
 	{
 		line = uCursorBox.at.posY;
@@ -3862,13 +3986,13 @@ UWidget::uFindUCS2 (const UCS2* text, int fromBegin)
 }
 
 int
-UWidget::uReplaceUCS2 (const UCS2* text)
+UWidget::uReplaceUCS2 (const UCS2* text, const int len)
 {
 	int	i;
 
 	if (editable!=EDITABLE) return -1;
 	uDeleteSelection ();
-	if (text!=0 && text[0] != 0) uInsertUCS2 (text);
+	if (text!=0) uInsertUCS2 (text, len);
 	return 0;
 }
 
