@@ -1,9 +1,13 @@
-//  UXKinput.cpp version 1.1
+//  UXKinput.cpp version 1.5
 //  yudit package - Unicode Editor for the X Window System (and Linux) 
 //
 //  Author: gsinai@iname.com (Gaspar Sinai)
-//  GNU Copyright (C) 1997,1998  Gaspar Sinai
+//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
 // 
+//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
+//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
 //  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
@@ -41,7 +45,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#if defined(__alpha) || (defined(_MIPS_SZLONG) && _MIPS_SZLONG == 64)
+#if defined(__alpha) || (defined(_MIPS_SZLONG) && _MIPS_SZLONG == 64) || defined(_AIX)
 typedef int int32;
 typedef unsigned int uint32; 
 #else
@@ -222,6 +226,8 @@ UXKinput::propertyChangeHandler (const XEvent* ev)
 	unsigned long		rest;
 	unsigned char*		propvalue;
 	void*			cludge;
+	UCS2*			instr;
+	int			instrLen;
 
 
 	// check event type
@@ -293,13 +299,12 @@ UXKinput::propertyChangeHandler (const XEvent* ev)
 	buffer = new unsigned char[propsize+4];
 	memcpy (buffer, propvalue, propsize);
 	buffer[propsize] = 0;
-	buffer[propsize+1] = 0;
-	buffer[propsize+2] = 0;
-	buffer[propsize+3] = 0xff;
 
-	uString->putString (buffer);
+	uString->putString (buffer, propsize);
 	uwidget->uDeleteSelection ();
-	uwidget->uInsertUCS2 ((const UCS2*) *uString);
+	instr = uString->getUString (&instrLen);
+	uwidget->uInsertUCS2 (instr, instrLen);
+	delete instr;
 	uwidget->uSetCursorViewable();
 
 	delete buffer;
